@@ -50,7 +50,7 @@ extern int pnum(char*);
 
 OENTRIES* find_opcode2(CSOUND*, char*);
 char* resolve_opcode_get_outarg(CSOUND* csound,
-                               OENTRIES* entries, char* inArgTypes);
+                                OENTRIES* entries, char* inArgTypes);
 int check_out_args(CSOUND* csound, char* outArgsFound, char* opOutArgs);
 char* get_arg_string_from_tree(CSOUND* csound, TREE* tree,
                                TYPE_TABLE* typeTable);
@@ -70,7 +70,7 @@ char* cs_strdup(CSOUND* csound, char* str) {
     retVal = csound->Malloc(csound, len + 1);
 
     if (len > 0) {
-      strncpy(retVal, str, len);
+      memcpy(retVal, str, len);
     }
     retVal[len] = '\0';
 
@@ -125,6 +125,8 @@ char* get_expression_opcode_type(CSOUND* csound, TREE* tree) {
       return "##not";
     case T_ARRAY:
       return "##array_get";
+    case S_ADDIN:
+      return "##addin";
     }
     csound->Warning(csound, Str("Unknown function type found: %d [%c]\n"),
                     tree->type, tree->type);
@@ -182,19 +184,23 @@ char* get_array_sub_type(CSOUND* csound, char* arrayName) {
 char* create_array_arg_type(CSOUND* csound, CS_VARIABLE* arrayVar) {
 
     int i, len = arrayVar->dimensions + 3;
-    char* retVal = csound->Malloc(csound, len);
-    retVal[len - 1] = '\0';
-    retVal[len - 2] = ']';
-    retVal[len - 3] = *arrayVar->subType->varTypeName;
-    for (i = len - 4; i >= 0; i--) {
+    if (arrayVar->subType!=NULL) {
+      char* retVal = csound->Malloc(csound, len);
+      retVal[len - 1] = '\0';
+      retVal[len - 2] = ']';
+      retVal[len - 3] = *arrayVar->subType->varTypeName;
+      for (i = len - 4; i >= 0; i--) {
         retVal[i] = '[';
+      }
+      return retVal;
     }
-    return retVal;
+    else
+      return NULL;
 }
 
 /* this checks if the annotated type exists */
 char *check_annotated_type(CSOUND* csound, OENTRIES* entries,
-                              char* outArgTypes) {
+                           char* outArgTypes) {
     int i;
     for (i = 0; i < entries->count; i++) {
       OENTRY* temp = entries->entries[i];
@@ -326,7 +332,7 @@ char* get_arg_type2(CSOUND* csound, TREE* tree, TYPE_TABLE* typeTable)
         if (tree->right->type == T_ARRAY &&
             tree->right->left->type == T_IDENT &&
             isirate(/*csound,*/ tree->right->right)) {
-          //printf("OK array case\n");
+          synterr(csound, Str("Use of i() with array element ill formed\n"));
         }
         else
           if (UNLIKELY(tree->right->type != LABEL_TOKEN))
@@ -389,8 +395,8 @@ char* get_arg_type2(CSOUND* csound, TREE* tree, TYPE_TABLE* typeTable)
         len2 = strlen(argTypeRight);
         inArgTypes = csound->Malloc(csound, len1 + len2 + 1);
 
-        strncpy(inArgTypes, argTypeLeft, len1);
-        strncpy(inArgTypes + len1, argTypeRight, len2);
+        memcpy(inArgTypes, argTypeLeft, len1);
+        memcpy(inArgTypes + len1, argTypeRight, len2);
 
         inArgTypes[len1 + len2] = '\0';
 
@@ -423,53 +429,53 @@ char* get_arg_type2(CSOUND* csound, TREE* tree, TYPE_TABLE* typeTable)
         return get_arg_type2(csound, tree->left, typeTable);
       }
       else {
-      char* argTypeLeft = get_arg_type2(csound, tree->left, typeTable);
-      char* argTypeRight = get_arg_type2(csound, tree->right, typeTable);
+        char* argTypeLeft = get_arg_type2(csound, tree->left, typeTable);
+        char* argTypeRight = get_arg_type2(csound, tree->right, typeTable);
 
-      char* opname = get_boolean_expression_opcode_type(csound, tree);
-      int len1, len2;
-      char* inArgTypes;
-      char* out;
-      OENTRIES* entries;
+        char* opname = get_boolean_expression_opcode_type(csound, tree);
+        int len1, len2;
+        char* inArgTypes;
+        char* out;
+        OENTRIES* entries;
 
-      if (UNLIKELY(argTypeLeft == NULL || argTypeRight == NULL)) {
-        synterr(csound,
-                Str("Unable to verify arg types for boolean expression '%s'\n"
-                    "Line %d\n"),
-                opname, tree->line);
-        do_baktrace(csound, tree->locn);
-        return NULL;
-      }
+        if (UNLIKELY(argTypeLeft == NULL || argTypeRight == NULL)) {
+          synterr(csound,
+                  Str("Unable to verify arg types for boolean expression '%s'\n"
+                      "Line %d\n"),
+                  opname, tree->line);
+          do_baktrace(csound, tree->locn);
+          return NULL;
+        }
 
-      entries = find_opcode2(csound, opname);
+        entries = find_opcode2(csound, opname);
 
-      len1 = strlen(argTypeLeft);
-      len2 = strlen(argTypeRight);
-      inArgTypes = csound->Malloc(csound, len1 + len2 + 1);
+        len1 = strlen(argTypeLeft);
+        len2 = strlen(argTypeRight);
+        inArgTypes = csound->Malloc(csound, len1 + len2 + 1);
 
-      strncpy(inArgTypes, argTypeLeft, len1);
-      strncpy(inArgTypes + len1, argTypeRight, len2);
+        memcpy(inArgTypes, argTypeLeft, len1);
+        memcpy(inArgTypes + len1, argTypeRight, len2);
 
-      inArgTypes[len1 + len2] = '\0';
+        inArgTypes[len1 + len2] = '\0';
 
-      out = resolve_opcode_get_outarg(csound, entries, inArgTypes);
-      csound->Free(csound, entries);
+        out = resolve_opcode_get_outarg(csound, entries, inArgTypes);
+        csound->Free(csound, entries);
 
-      if (UNLIKELY(out == NULL)) {
-        synterr(csound, Str("error: boolean expression '%s' with arg "
-                            "types %s not found, line %d\n"),
-                opname, inArgTypes, tree->line);
-        do_baktrace(csound, tree->locn);
+        if (UNLIKELY(out == NULL)) {
+          synterr(csound, Str("error: boolean expression '%s' with arg "
+                              "types %s not found, line %d\n"),
+                  opname, inArgTypes, tree->line);
+          do_baktrace(csound, tree->locn);
+          csound->Free(csound, inArgTypes);
+          return NULL;
+        }
+
+        csound->Free(csound, argTypeLeft);
+        csound->Free(csound, argTypeRight);
         csound->Free(csound, inArgTypes);
-        return NULL;
+        return cs_strdup(csound, out);
+
       }
-
-      csound->Free(csound, argTypeLeft);
-      csound->Free(csound, argTypeRight);
-      csound->Free(csound, inArgTypes);
-      return cs_strdup(csound, out);
-
-    }
     }
 
     switch(tree->type) {
@@ -545,13 +551,19 @@ char* get_arg_type2(CSOUND* csound, TREE* tree, TYPE_TABLE* typeTable)
       if (UNLIKELY(var == NULL)) {
         synterr(csound, Str("Variable '%s' used before defined\n"
                             "Line %d\n"),
-                tree->value->lexeme, tree->line);
+                tree->value->lexeme, tree->line-1); /* -1 as read next line! */
         do_baktrace(csound, tree->locn);
         return NULL;
       }
 
       if (var->varType == &CS_VAR_TYPE_ARRAY) {
-        return create_array_arg_type(csound, var);
+        char *res = create_array_arg_type(csound, var);
+        if (res==NULL) {        /* **REVIEW** this double syntax error */
+          synterr(csound, Str("Array of unknown type\n"));
+          csoundMessage(csound, Str("Line: %d\n"), tree->line-1);
+          do_baktrace(csound, tree->locn);
+        }
+        return res;
       } else {
         return cs_strdup(csound, var->varType->varTypeName);
       }
@@ -762,6 +774,9 @@ int check_in_args(CSOUND* csound, char* inArgsFound, char* opInArgs) {
       if (argsRequired == NULL) {
         return 0;
       }
+      if (argsFoundCount>=VARGMAX) {
+        return -1;
+      }
 
       if ((argsFoundCount > argsRequiredCount) &&
           !(is_in_var_arg(*argsRequired[argsRequiredCount - 1]))) {
@@ -964,8 +979,7 @@ OENTRY* resolve_opcode(CSOUND* csound, OENTRIES* entries,
                        char* outArgTypes, char* inArgTypes) {
 
 //    OENTRY* retVal = NULL;
-    int i;
-
+  int i, check;
 
     for (i = 0; i < entries->count; i++) {
         OENTRY* temp = entries->entries[i];
@@ -975,12 +989,18 @@ OENTRY* resolve_opcode(CSOUND* csound, OENTRIES* entries,
 //            }
 //            continue;
 //        }
-        if (check_in_args(csound, inArgTypes, temp->intypes) &&
+        if ((check = check_in_args(csound, inArgTypes, temp->intypes)) &&
             check_out_args(csound, outArgTypes, temp->outypes)) {
 //            if (retVal != NULL) {
 //                return NULL;
 //            }
 //            retVal = temp;
+          if (check == -1)
+              synterr(csound,
+                      Str("Found %d inputs for %s which is more than "
+                          "the %d allowed\n"),
+                      argsRequired(inArgTypes), temp->opname, VARGMAX);
+
             return temp;
         }
     }
@@ -991,15 +1011,18 @@ OENTRY* resolve_opcode(CSOUND* csound, OENTRIES* entries,
 OENTRY* resolve_opcode_exact(CSOUND* csound, OENTRIES* entries,
                        char* outArgTypes, char* inArgTypes) {
     IGN(csound);
+    OENTRY* retVal = NULL;
     int i;
+
+    char* outTest = (!strcmp("0", outArgTypes)) ? "" : outArgTypes;
     for (i = 0; i < entries->count; i++) {
       OENTRY* temp = entries->entries[i];
       if (temp->intypes != NULL && !strcmp(inArgTypes, temp->intypes) &&
-          temp->outypes != NULL && !strcmp(outArgTypes, temp->outypes)) {
-        return temp;
+          temp->outypes != NULL && !strcmp(outTest, temp->outypes)) {
+        retVal = temp;
       }
     }
-    return NULL;
+    return retVal;
 }
 
 /* used when creating T_FUNCTION's */
@@ -1022,33 +1045,6 @@ char* resolve_opcode_get_outarg(CSOUND* csound, OENTRIES* entries,
     return NULL;
 }
 
-//PUBLIC int resolve_opcode_num(CSOUND* csound, OENTRIES* entries,
-//                              char* outArgTypes, char* inArgTypes) {
-//
-//    int i;
-////    int retVal = -1;
-//
-//    for (i = 0; i < entries->count; i++) {
-//        OENTRY* temp = entries->entries[i];
-//        if (temp->intypes == NULL && temp->outypes == NULL) {
-//            continue;
-//        }
-//        if (check_in_args(csound, inArgTypes, temp->intypes) &&
-//            check_out_args(csound, outArgTypes, temp->outypes)) {
-////            if (retVal >= 0) {
-////                return 0;
-////            }
-////            retVal = entries->opnum[i];
-//            return entries->opnum[i];
-//        }
-//
-//    }
-//
-////    return (retVal < 0) ? 0 : retVal;
-//    return 0;
-//}
-
-
 /* Converts internal array specifier from [[a] to a[][].
  Used by get_arg_string_from_tree to create an arg string that is
  compatible with the ones found in OENTRY's.  splitArgs converts back
@@ -1499,6 +1495,10 @@ int verify_opcode(CSOUND* csound, TREE* root, TYPE_TABLE* typeTable) {
               opcodeName);
       csoundMessage(csound, Str("Found: %s %s %s\n"),
                     leftArgString, root->value->lexeme, rightArgString);
+      if (root->left && root->left->value && root->right && root->right->value)
+      csoundMessage(csound, Str("       %s %s %s ...\n"),
+                    root->left->value->lexeme, root->value->lexeme,
+                    root->right->value->lexeme);
       csoundMessage(csound, Str("Line: %d\n"),
                     root->line);
       do_baktrace(csound, root->locn);
@@ -1510,16 +1510,34 @@ int verify_opcode(CSOUND* csound, TREE* root, TYPE_TABLE* typeTable) {
       return 0;
     }
     else {
+      //fprintf(stderr, "left=%p\n", left);
+      //fprintf(stderr, "left->value=%p\n", left->value);
+      //fprintf(stderr, "left->value->lexeme=%p\n", left->value->lexeme);
+      //fprintf(stderr, "opname = %s\n", oentry->opname);
       if (csound->oparms->sampleAccurate &&
           (strcmp(oentry->opname, "=.a")==0) &&
-          left->value->lexeme[0]=='a') { /* Deal with sample accurate assigns */
+          (left!=NULL) && (left->value!=NULL) &&
+          (left->value->lexeme[0]=='a')) { /* Deal with sample accurate assigns */
         int i = 0;
         while (strcmp(entries->entries[i]->opname, "=.l")) {
-          printf("not %d %s\n",i, entries->entries[i]->opname);
+          //printf("not %d %s\n",i, entries->entries[i]->opname);
           i++;
         }
         oentry = entries->entries[i];
       }
+      else {
+        if (csound->oparms->sampleAccurate &&
+            (strcmp(oentry->opname, "=._")==0) &&
+            (left->value->lexeme[0]=='a'))
+          {
+            int i = 0;
+            while (strcmp(entries->entries[i]->opname, "=.L")) {
+              //printf("not %d %s\n",i, entries->entries[i]->opname);
+              i++;
+            }
+            oentry = entries->entries[i];
+          }
+      }
       root->markup = oentry;
     }
     csound->Free(csound, leftArgString);
@@ -1937,8 +1955,8 @@ extern char *csound_orcget_current_pointer(void *);
 void csound_orcerror(PARSE_PARM *pp, void *yyscanner,
                      CSOUND *csound, TREE **astTree, const char *str)
 {
-     IGN(pp);
-      IGN(astTree);
+    IGN(pp);
+    IGN(astTree);
     char ch;
     char *p = csound_orcget_current_pointer(yyscanner)-1;
     int line = csound_orcget_lineno(yyscanner);
@@ -2062,7 +2080,7 @@ TREE* make_leaf(CSOUND *csound, int line, int locn, int type, ORCTOKEN *v)
     return ans;
 }
 
-static void delete_tree(CSOUND *csound, TREE *l)
+void delete_tree(CSOUND *csound, TREE *l)
 {
     while (1) {
       TREE *old = l;
@@ -2419,6 +2437,8 @@ static void print_tree_xml(CSOUND *csound, TREE *l, int n, int which)
                       l->value->lexeme); break;
     case S_ELIPSIS:
       csound->Message(csound,"name=\"S_ELIPSIS\""); break;
+    case S_ADDIN:
+      csound->Message(csound,"name=\"##addin\""); break;
 //    case T_MAPI:
 //      csound->Message(csound,"name=\"T_MAPI\""); break;
 //    case T_MAPK:
