@@ -24,14 +24,12 @@
 #include "entry1.h"             /*                      ENTRY1.C        */
 #include "interlocks.h"
 
-/* thread vals, where isub=1, ksub=2, asub=4:
+/* thread vals, where isub=1, ksub=2:
    0 =     1  OR   2  (B out only)
    1 =     1
    2 =             2
    3 =     1  AND  2
-   4 =                     4
-   5 =     1  AND          4
-   7 =     1  AND (2  OR   4)                              */
+ */
 
 /* inarg types include the following:
 
@@ -122,6 +120,8 @@ OENTRY opcodlst_1[] = {
   { "turnoff",S(LINK),0,    2,      "",     "",     NULL,   turnoff, NULL, NULL },
   {  "=.S",   S(STRCPY_OP),0,   1,  "S",    "S",
      (SUBR) strcpy_opcode_S, NULL, (SUBR) NULL, NULL    },
+  {  "#=.S",   S(STRCPY_OP),0,   2,  "S",    "S",
+     NULL, (SUBR) strcpy_opcode_S, (SUBR) NULL, NULL    },
   {  "=.T",   S(STRGET_OP),0,   1,  "S",    "i",
      (SUBR) strcpy_opcode_p, (SUBR) NULL, (SUBR) NULL, NULL                 },
   { "=.r",    S(ASSIGN),0,  1,      "r",    "i",    rassign, NULL, NULL, NULL },
@@ -160,7 +160,7 @@ OENTRY opcodlst_1[] = {
   { ":cond.i",     S(CONVAL),0,  1,      "i",    "bii",  conval                  },
   { ":cond.k",     S(CONVAL),0,  2,      "k",    "Bkk",  NULL,   conval          },
   { ":cond.a",     S(CONVAL),0,  2,      "a",    "Bxx",  NULL,   aconval },
-  { ":cond.s",     S(CONVAL),0,  3,      "S",    "bSS",  conval, NULL         },
+  { ":cond.s",     S(CONVAL),0,  1,      "S",    "bSS",  conval, NULL         },
   { ":cond.S",     S(CONVAL),0,  3,      "S",    "BSS",  conval, conval       },
   { "##add.ii",  S(AOP),0,    1,      "i",    "ii",   addkk                   },
   { "##sub.ii",  S(AOP),0,    1,      "i",    "ii",   subkk                   },
@@ -187,7 +187,15 @@ OENTRY opcodlst_1[] = {
   { "##mul.aa",  S(AOP),0,    2,      "a",    "aa",   NULL,   mulaa   },
   { "##div.aa",  S(AOP),0,    2,      "a",    "aa",   NULL,   divaa   },
   { "##mod.aa",  S(AOP),0,    2,      "a",    "aa",   NULL,   modaa   },
-  { "divz",   0xfffc                                                      },
+  { "##addin.i", S(ASSIGN),0, 1,      "i",    "i",    addin,  NULL    },
+  { "##addin.k", S(ASSIGN),0, 2,      "k",    "k",    NULL,   addin   },
+  { "##addin.K", S(ASSIGN),0, 2,      "a",    "k",    NULL,   addinak },
+  { "##addin.a", S(ASSIGN),0, 2,      "a",    "a",    NULL,   addina  },
+  { "##subin.i", S(ASSIGN),0, 1,      "i",    "i",    subin,  NULL    },
+  { "##subin.k", S(ASSIGN),0, 2,      "k",    "k",    NULL,   subin   },
+  { "##subin.K", S(ASSIGN),0, 2,      "a",    "k",    NULL,   subinak },
+  { "##subin.a", S(ASSIGN),0, 2,      "a",    "a",    NULL,   subina  },
+  //{ "divz",   0xfffc                                                      },
   { "divz.ii", S(DIVZ),0,   1,      "i",    "iii",  divzkk, NULL,   NULL    },
   { "divz.kk", S(DIVZ),0,   2,      "k",    "kkk",  NULL,   divzkk, NULL    },
   { "divz.ak", S(DIVZ),0,   2,      "a",    "akk",  NULL,   divzak  },
@@ -198,6 +206,7 @@ OENTRY opcodlst_1[] = {
   { "round.i",S(EVAL),0,    1,      "i",    "i",    int1_round              },
   { "floor.i",S(EVAL),0,    1,      "i",    "i",    int1_floor              },
   { "ceil.i", S(EVAL),0,    1,      "i",    "i",    int1_ceil               },
+  { "rndseed", S(EVAL),0,    1,      "",    "i",    rnd1seed                },
   { "rnd.i",  S(EVAL),0,    1,      "i",    "i",    rnd1                    },
   { "birnd.i",S(EVAL),0,    1,      "i",    "i",    birnd1                  },
   { "abs.i",  S(EVAL),0,    1,      "i",    "i",    abs1                    },
@@ -367,41 +376,35 @@ OENTRY opcodlst_1[] = {
   { "signum.i", S(ASSIGN), 0, 1,     "i",   "i", signum, NULL, NULL       },
   { "signum.k", S(ASSIGN), 0, 3,     "k",   "k", signum, signum, NULL     },
   { "signum.a", S(ASSIGN), 0, 2,     "a",   "a", NULL, asignum      },
-  /* { "table",  0xffff, TR                                                   }, */
-  /* { "tablei", 0xffff, TR                                                   }, */
-  /* { "table3", 0xffff, TR                                                   }, */
-  { "table.i",  S(TABL),0, 1,      "i",    "iiooo",(SUBR)tabler_init       },
-  { "table.k",  S(TABL),0, 3,      "k",    "xiooo",(SUBR)tabl_setup,
+  { "table.i",  S(TABL),TR, 1,      "i",    "iiooo",(SUBR)tabler_init       },
+  { "table.k",  S(TABL),TR, 3,      "k",    "xiooo",(SUBR)tabl_setup,
     (SUBR)tabler_kontrol        },
-  { "table.a",  S(TABL),0, 3,      "a",    "xiooo",(SUBR)tabl_setup,
+  { "table.a",  S(TABL),TR, 3,      "a",    "xiooo",(SUBR)tabl_setup,
     (SUBR)tabler_audio                                                      },
-  { "tablei.i", S(TABL),0, 1,      "i",    "iiooo",(SUBR)tableir_init       },
-  { "tablei.k", S(TABL),0, 3,      "k",    "xiooo",(SUBR)tabl_setup,
+  { "tablei.i", S(TABL),TR, 1,      "i",    "iiooo",(SUBR)tableir_init      },
+  { "tablei.k", S(TABL),TR, 3,      "k",    "xiooo",(SUBR)tabl_setup,
     (SUBR)tableir_kontrol                                                   },
-  { "tablei.a", S(TABL),0, 3,      "a",    "xiooo",(SUBR)tabl_setup,
+  { "tablei.a", S(TABL),TR, 3,      "a",    "xiooo",(SUBR)tabl_setup,
     (SUBR)tableir_audio                                                     },
-  { "table3.i", S(TABL),0, 1,      "i",    "iiooo",(SUBR)table3r_init       },
-  { "table3.k", S(TABL),0, 3,      "k",    "xiooo",(SUBR)tabl_setup,
+  { "table3.i", S(TABL),TR, 1,      "i",    "iiooo",(SUBR)table3r_init      },
+  { "table3.k", S(TABL),TR, 3,      "k",    "xiooo",(SUBR)tabl_setup,
     (SUBR)table3r_kontrol                                                   },
-  { "table3.a", S(TABL),0, 3,      "a",    "xiooo",(SUBR)tabl_setup,
+  { "table3.a", S(TABL),TR, 3,      "a",    "xiooo",(SUBR)tabl_setup,
     (SUBR)table3r_audio                                                     },
-  /* { "ptable",  0xffff, TR                                                  }, */
-  /* { "ptablei", 0xffff, TR                                                  }, */
-  /* { "ptable3", 0xffff, TR                                                  }, */
-  { "ptable.i",  S(TABLE),0, 1,     "i",    "iiooo",(SUBR)tabler_init       },
-  { "ptable.k",  S(TABLE),0, 3,     "k",    "xiooo",(SUBR)tabl_setup,
+  { "ptable.i",  S(TABLE),TR|_QQ, 1,"i",    "iiooo",(SUBR)tabler_init       },
+  { "ptable.k",  S(TABLE),TR|_QQ, 3,     "k",    "xiooo",(SUBR)tabl_setup,
     (SUBR)tabler_kontrol                                                    },
-  { "ptable.a",  S(TABLE),0, 3,     "a",    "xiooo",(SUBR)tabl_setup,
+  { "ptable.a",  S(TABLE),TR|_QQ, 3,     "a",    "xiooo",(SUBR)tabl_setup,
     (SUBR)tabler_audio                                                      },
-  { "ptablei.i", S(TABLE),0, 1,     "i",    "iiooo",(SUBR)tableir_init      },
-  { "ptablei.k", S(TABLE),0, 3,     "k",    "xiooo",(SUBR)tabl_setup,
+  { "ptablei.i", S(TABLE),TR|_QQ, 1,"i",    "iiooo",(SUBR)tableir_init      },
+  { "ptablei.k", S(TABLE),TR|_QQ, 3,     "k",    "xiooo",(SUBR)tabl_setup,
     (SUBR)tableir_kontrol                                                   },
-  { "ptablei.a", S(TABLE),0, 3,     "a",    "xiooo",(SUBR)tabl_setup,
+  { "ptablei.a", S(TABLE),TR|_QQ, 3,     "a",    "xiooo",(SUBR)tabl_setup,
     (SUBR)tableir_audio                                                     },
-  { "ptable3.i", S(TABLE),0, 1,     "i",    "iiooo",(SUBR)table3r_init      },
-  { "ptable3.k", S(TABLE),0, 3,     "k",    "xiooo",(SUBR)tabl_setup,
+  { "ptable3.i", S(TABLE),TR|_QQ, 1,"i",    "iiooo",(SUBR)table3r_init      },
+  { "ptable3.k", S(TABLE),TR|_QQ, 3,     "k",    "xiooo",(SUBR)tabl_setup,
     (SUBR)table3r_kontrol                                                   },
-  { "ptable3.a", S(TABLE),0, 3,     "a",    "xiooo",(SUBR)tabl_setup,
+  { "ptable3.a", S(TABLE),TR|_QQ, 3,     "a",    "xiooo",(SUBR)tabl_setup,
     (SUBR)table3r_audio         },
   { "oscil1", S(OSCIL1), TR, 3,     "k",    "ikij", ko1set, kosc1          },
   { "oscil1i",S(OSCIL1), TR, 3,     "k",    "ikij", ko1set, kosc1i         },
@@ -461,8 +464,10 @@ OENTRY opcodlst_1[] = {
   { "rand.k",   S(RAND),0,    3,      "k",    "xvoo", rndset, krand,  NULL  },
   { "randh",  S(RANDH),0,   3,      "a",    "xxvoo", rhset, randh   },
   { "randh.k",  S(RANDH),0,   3,      "k",    "xxvoo", rhset, krandh, NULL   },
-  { "randi",  S(RANDI),0,   3,      "a",    "xxvoo", riset, randi   },
-  { "randi.k",  S(RANDI),0,   3,      "k",    "xxvoo", riset, krandi, NULL  },
+  { "randi",  S(RANDI),0,   3,      "a",    "xxvoo", riset, randi     },
+  { "randi.k",  S(RANDI),0, 3,      "k",    "xxvoo", riset, krandi    },
+  { "randc",  S(RANDC),0,   3,      "a",    "xxvoo", rcset, randc     },
+  { "randc.k",  S(RANDC),0, 3,      "k",    "xxvoo", rcset, krandc    },
   { "port",   S(PORT),0,    3,      "k",    "kio",  porset, port            },
   { "tone.k", S(TONE),0,    3,      "a",    "ako",  tonset,   tone    },
   { "tonex.k",S(TONEX),0,   3,      "a",    "akoo", tonsetx,  tonex   },
@@ -496,20 +501,20 @@ OENTRY opcodlst_1[] = {
   { "in.A",   S(INA),0,     2,      "a[]",  "",     NULL,   inarray },
   { "ins",    S(INS),0,     2,      "aa",   "",     NULL,   ins     },
   { "inq",    S(INQ),0,     2,      "aaaa", "",     NULL,   inq     },
-  { "out.a",  S(OUTX),0,     3,      "",     "y",    ochn,   outall },
-  { "out.A",  S(OUTARRAY),0, 3,      "",     "a[]",  outarr_init,  outarr },
-  { "outs",   S(OUTX),0,     3,      "",     "y",    ochn,   outall },
-  { "outq",   S(OUTX),0,     3,      "",     "y",    ochn,   outall },
-  { "outh",   S(OUTX),0,     3,      "",     "y",    ochn,   outall },
-  { "outo",   S(OUTX),0,     3,      "",     "y",    ochn,   outall },
-  { "outx",   S(OUTX),0,     3,      "",     "y",    ochn,   outall },
-  { "out32",  S(OUTX),0,     3,      "",     "y",    ochn,   outall },
-  { "outs1",  S(OUTM),0,    2,      "",     "a",    NULL,   outs1   },
-  { "outs2",  S(OUTM),0,    3,      "",     "a",    och2,   outs2   },
-  { "outq1",  S(OUTM),0,    2,      "",     "a",    NULL,   outs1   },
-  { "outq2",  S(OUTM),0,    3,      "",     "a",    och2,   outs2   },
-  { "outq3",  S(OUTM),0,    3,      "",     "a",    och3,   outq3   },
-  { "outq4",  S(OUTM),0,    3,      "",     "a",    och2,   outq4   },
+  { "out.a",  S(OUTX),IR,     3,      "",     "y",    ochn,   outall },
+  { "out.A",  S(OUTARRAY),IR, 3,      "",     "a[]",  outarr_init,  outarr },
+  { "outs",   S(OUTX),IR,     3,      "",     "y",    ochn,   outall },
+  { "outq",   S(OUTX),IR,     3,      "",     "y",    ochn,   outall },
+  { "outh",   S(OUTX),IR,     3,      "",     "y",    ochn,   outall },
+  { "outo",   S(OUTX),IR,     3,      "",     "y",    ochn,   outall },
+  { "outx",   S(OUTX),IR,     3,      "",     "y",    ochn,   outall },
+  { "out32",  S(OUTX),IR,     3,      "",     "y",    ochn,   outall },
+  { "outs1",  S(OUTM),IR,    2,      "",     "a",    NULL,   outs1   },
+  { "outs2",  S(OUTM),IR,    3,      "",     "a",    och2,   outs2   },
+  { "outq1",  S(OUTM),IR,    2,      "",     "a",    NULL,   outs1   },
+  { "outq2",  S(OUTM),IR,    3,      "",     "a",    och2,   outs2   },
+  { "outq3",  S(OUTM),IR,    3,      "",     "a",    och3,   outq3   },
+  { "outq4",  S(OUTM),IR,    3,      "",     "a",    och2,   outq4   },
   { "igoto",  S(GOTO),0,    1,      "",     "l",    igoto                   },
   { "kgoto",  S(GOTO),0,    2,      "",     "l",    NULL,   kgoto           },
   { "goto",   S(GOTO),0,    3,      "",     "l",    igoto,  kgoto           },
@@ -580,6 +585,9 @@ OENTRY opcodlst_1[] = {
   { "gauss.i" , S(PRAND),0, 1,      "i",    "k",    ikgaus,  NULL,  NULL    },
   { "gauss.k" , S(PRAND),0, 2,      "k",    "k",    NULL, ikgaus,   NULL    },
   { "gauss.a" , S(PRAND),0, 2,      "a",    "k",    NULL,  agaus   },
+  { "gauss.iii" , S(GAUSS),0, 1,      "i",    "ii",    gauss_scalar,  NULL,  NULL    },
+  { "gauss.kkk" , S(GAUSS),0, 2,      "k",    "kk",    NULL, gauss_scalar,   NULL    },
+  { "gauss.akk" , S(GAUSS),0, 2,      "a",    "kk",    NULL, gauss_vector   },
   { "weibull.i",S(PRAND),0, 1,      "i",    "kk",   ikweib,  NULL,  NULL    },
   { "weibull.k",S(PRAND),0, 2,      "k",    "kk",   NULL, ikweib,   NULL    },
   { "weibull.a",S(PRAND),0, 2,      "a",    "kk",   NULL,  aweib   },
@@ -672,12 +680,12 @@ OENTRY opcodlst_1[] = {
   { "filepeak.i", S(SNDINFOPEAK),0, 1, "i",   "io",   filepeak, NULL, NULL       },
   { "filevalid.i", S(FILEVALID),0, 1,  "i",   "i",    filevalid, NULL, NULL      },
   /*  { "nlalp", S(NLALP),0,     3,     "a",  "akkoo", nlalp_set, nlalp }, */
-  { "ptableiw",  S(TABLEW),TW, 1,   "", "iiiooo", (SUBR)tablew_init, NULL, NULL},
-  { "ptablew.kk", S(TABLEW),0,  3,  "", "kkiooo",(SUBR)tabl_setup,
+  { "ptableiw",  S(TABLEW),TW|_QQ, 1, "", "iiiooo", (SUBR)tablew_init, NULL, NULL},
+  { "ptablew.kk", S(TABLEW),TB,  3,  "", "kkiooo",(SUBR)tabl_setup,
     (SUBR)tablew_kontrol, NULL          },
-  { "ptablew.aa", S(TABLEW),0,  3,  "", "aaiooo",(SUBR)tabl_setup,
+  { "ptablew.aa", S(TABLEW),TB,  3,  "", "aaiooo",(SUBR)tabl_setup,
     (SUBR)tablew_audio               },
-  { "tableiw",  S(TABL),TW, 1,    "",   "iiiooo", (SUBR)tablew_init, NULL, NULL},
+  { "tableiw",  S(TABL),TW|_QQ, 1, "",   "iiiooo", (SUBR)tablew_init, NULL, NULL},
   { "tablew",  S(TABL),TW, 1,    "",   "iiiooo", (SUBR)tablew_init, NULL, NULL},
   { "tablew.kk", S(TABL),TW,  3,    "", "kkiooo",(SUBR)tabl_setup,
     (SUBR)tablew_kontrol, NULL          },
@@ -714,23 +722,8 @@ OENTRY opcodlst_1[] = {
     NULL         },
   { "table3kt.a", S(TABL),TR, 3,  "a",  "xkooo", (SUBR)tablkt_setup,
     (SUBR)table3rkt_audio          },
-  { "zakinit", S(ZAKINIT), ZB, 1,  "",   "ii",   (SUBR)zakinit, NULL,  NULL      },
-  { "zir",    S(ZKR),ZR,  1,   "i",  "i",    (SUBR)zir,     NULL,  NULL      },
-  { "zkr",    S(ZKR),ZR,  3,   "k",  "k",    (SUBR)zkset,   (SUBR)zkr,   NULL},
-  { "ziw",    S(ZKW),ZW, 1,   "",   "ii",   (SUBR)ziw,     NULL,  NULL      },
-  { "zkw",    S(ZKW),     ZW, 3,   "",   "kk",   (SUBR)zkset,   (SUBR)zkw,   NULL},
-  { "ziwm",   S(ZKWM),    ZB, 1,   "",   "iip",  (SUBR)ziwm,    NULL,  NULL      },
-  { "zkwm",   S(ZKWM),    ZB, 3,   "",   "kkp",  (SUBR)zkset,   (SUBR)zkwm,  NULL},
-  { "zkmod",  S(ZKMOD),   ZB, 2,   "k",  "kk",   NULL,    (SUBR)zkmod, NULL      },
-  { "zkcl",   S(ZKCL),    ZW, 3,   "",  "kk",   (SUBR)zkset,   (SUBR)zkcl,  NULL },
-  { "zar",    S(ZAR),ZR,  3,   "a", "k",    (SUBR)zaset,  (SUBR)zar  },
-  { "zarg",   S(ZARG),   ZB, 3,   "a", "kk",   (SUBR)zaset,  (SUBR)zarg },
-  { "zaw",    S(ZAW),    ZW, 3,   "",  "ak",   (SUBR)zaset,  (SUBR)zaw  },
-  { "zawm",   S(ZAWM),   ZB, 3,   "",  "akp",  (SUBR)zaset,  (SUBR)zawm },
-  { "zamod",  S(ZAMOD),  ZB, 2,   "a", "ak",   NULL,  (SUBR)zamod},
-  { "zacl",   S(ZACL),   ZW, 3,   "",  "kk",   (SUBR)zaset,  (SUBR)zacl},
-  { "inz",    S(IOZ),    ZW, 2,   "",   "k",    (SUBR)zaset,   (SUBR)inz  },
-  { "outz",   S(IOZ),ZR,  2,   "",   "k",    (SUBR)zaset,   (SUBR)outz },
+  { "inz",    S(IOZ),    ZW, 2,   "",   "k",  NULL,   (SUBR)inz  },
+  { "outz",   S(IOZ),ZR|IR,  2,   "",   "k",    NULL,   (SUBR)outz },
   { "timek.i", S(RDTIME),0, 1,   "i",  "",     (SUBR)timek,   NULL,  NULL },
   { "times.i", S(RDTIME),0, 1,   "i",  "",     (SUBR)timesr,  NULL,  NULL },
   { "timek.k",  S(RDTIME),0, 2,  "k",  "",     NULL,    (SUBR)timek, NULL },
@@ -743,13 +736,13 @@ OENTRY opcodlst_1[] = {
   { "peak.a",   S(PEAK),0,  2,   "k",  "a",    NULL,     (SUBR)peaka   },
   { "printk", S(PRINTK),WR,  3,"",     "ikoooo",
     (SUBR)printkset, (SUBR)printk, NULL },
-  { "printks",S(PRINTKS),WR, 3,   "",   "SiM",
+  { "printks",S(PRINTKS),WR, 3,   "",   "SiN",
     (SUBR)printksset_S,(SUBR)printks, NULL },
   { "printks2", sizeof(PRINTK3),0, 3, "", "Sk", (SUBR)printk3set, (SUBR)printk3 },
-  { "printks.i",S(PRINTKS),WR, 3,   "",   "iiM",
+  { "printks.i",S(PRINTKS),WR, 3,   "",   "iiN",
     (SUBR)printksset,(SUBR)printks, NULL },
-  { "prints",S(PRINTS),0,   1,   "",   "SM",   (SUBR)printsset_S, NULL, NULL },
-  { "prints.i",S(PRINTS),0,   1,   "",   "iM",   (SUBR)printsset, NULL, NULL },
+  { "prints",S(PRINTS),0,   1,   "",   "SN",   (SUBR)printsset_S, NULL, NULL },
+  { "prints.i",S(PRINTS),0,   1,   "",   "iN",   (SUBR)printsset, NULL, NULL },
   { "printk2", S(PRINTK2), WR, 3, "",   "koo",
     (SUBR)printk2set, (SUBR)printk2, NULL },
   { "portk",  S(KPORT),0,   3, "k",     "kko",  (SUBR)kporset, (SUBR)kport, NULL },
@@ -779,8 +772,8 @@ OENTRY opcodlst_1[] = {
      ***BEWARE***
      CODE REMOVED 2011-Dec-14
   */
-  { "outch",  S(OUTCH),0,   2,      "",     "Z",    NULL,   outch   },
-  { "outc",   S(OUTX),0,    2,      "",     "y",    ochn,   outall  },
+  { "outch",  S(OUTCH),IR,   2,      "",     "Z",    NULL,   outch   },
+  { "outc",   S(OUTX),IR,    2,      "",     "y",    ochn,   outall  },
   { "cpsxpch", S(XENH),TR, 1,      "i",    "iiii", cpsxpch, NULL,  NULL    },
   { "cps2pch", S(XENH),TR, 1,      "i",    "ii",   cps2pch, NULL,  NULL    },
   { "cpstun", S(CPSTUN),  TR, 2,      "k",    "kkk",   NULL,   cpstun         },
@@ -802,6 +795,21 @@ OENTRY opcodlst_1[] = {
     schedule_S, NULL, NULL },
   { "schedule.SN", S(SCHED),0,  1,     "",     "SiiN",
     schedule_SN, NULL, NULL },
+  { "schedule.array", S(SCHED),0,  1,     "",     "i[]",
+    schedule_array, NULL, NULL },
+  /* **** Experimental schedulek opcodes **** */
+  { "schedulek",   S(SCHED),0,  2,     "",     "kkkM",
+    NULL, schedule, NULL },
+  { "schedulek.N", S(SCHED),0,  2,     "",     "kkkN",
+    NULL, schedule_N, NULL },
+  { "schedulek.S", S(SCHED),0,  2,     "",     "SkkM",
+    NULL, schedule_S, NULL },
+  { "schedulek.SN", S(SCHED),0, 2,     "",     "SkkN",
+    NULL, schedule_SN, NULL },
+  { "schedulek.array",   S(SCHED),0,  2,     "",     "k[]",
+    NULL, schedule_array, NULL },
+  
+  /* **** End of schedulek **** */
   { "schedwhen", S(WSCHED),0,3,     "",     "kkkkm",ifschedule, kschedule, NULL },
   { "schedwhen", S(WSCHED),0,3,     "",     "kSkkm",ifschedule, kschedule, NULL },
   { "schedkwhen", S(TRIGINSTR),0, 3,"",     "kkkkkz",triginset, ktriginstr, NULL },
@@ -833,24 +841,18 @@ OENTRY opcodlst_1[] = {
     (SUBR)reverbx_set,(SUBR)reverbx  },
   { "nreverb",  S(NREV2),0,  3,     "a",    "akkoojoj",
     (SUBR)reverbx_set,(SUBR) reverbx },
-  { "=.f",      S(FASSIGN),0, 3,    "f",   "f",
-    (SUBR)fassign_set, (SUBR)fassign },
-  { "init.f",   S(FASSIGN),0, 1,    "f",   "f",
-    (SUBR)fassign_set, NULL, NULL    },
-  { "pvsanal",  S(PVSANAL), 0, 3,   "f",   "aiiiioo",
-    pvsanalset, pvsanal   },
+  { "=.f",      S(FASSIGN),0, 3,    "f",   "f", (SUBR)fassign_set, (SUBR)fassign },
+  { "init.f",   S(FASSIGN),0, 1,    "f",   "f", (SUBR)fassign_set, NULL, NULL    },
+  { "pvsanal",  S(PVSANAL), 0, 3,   "f",   "aiiiioo", pvsanalset, pvsanal   },
   { "pvsynth",  S(PVSYNTH),0, 3,    "a",   "fo",     pvsynthset, pvsynth },
   { "pvsadsyn", S(PVADS),0,   3,    "a",   "fikopo", pvadsynset, pvadsyn, NULL },
   { "pvscross", S(PVSCROSS),0,3,    "f",   "ffkk",   pvscrosset, pvscross, NULL },
   { "pvsfread", S(PVSFREAD),0,3,    "f",   "kSo",    pvsfreadset_S, pvsfread, NULL},
-  { "pvsfread.i", S(PVSFREAD),0,3,    "f",   "kio",    pvsfreadset, pvsfread, NULL},
+  { "pvsfread.i", S(PVSFREAD),0,3,  "f",   "kio",    pvsfreadset, pvsfread, NULL},
   { "pvsmaska", S(PVSMASKA),0,3,    "f",   "fik",    pvsmaskaset, pvsmaska, NULL  },
-  { "pvsftw",   S(PVSFTW),  TW, 3,    "k",   "fio",    pvsftwset, pvsftw, NULL  },
+  { "pvsftw",   S(PVSFTW),  TW, 3,  "k",   "fio",    pvsftwset, pvsftw, NULL  },
   { "pvsftr",   S(PVSFTR),TR, 3,    "",    "fio",    pvsftrset, pvsftr, NULL  },
   { "pvsinfo",  S(PVSINFO),0, 1,    "iiii","f",      pvsinfo, NULL, NULL    },
-  { "octave",   0xffff                                                      },
-  { "semitone", 0xffff                                                      },
-  { "cent",     0xffff                                                      },
   { "octave.i", S(EVAL),0,    1,    "i",    "i",     powoftwo               },
   { "octave.k", S(EVAL),0,    2,    "k",    "k",     NULL,  powoftwo        },
   { "octave.a", S(EVAL),0,    2,    "a",    "a",     NULL, powoftwoa  },
@@ -860,13 +862,12 @@ OENTRY opcodlst_1[] = {
   { "cent.i",   S(EVAL),0,    1,    "i",    "i",     cent                   },
   { "cent.k",   S(EVAL),0,    2,    "k",    "k",     NULL,  cent            },
   { "cent.a",   S(EVAL),0,    2,    "a",    "a",     NULL, acent      },
-  { "db",       0xffff                                                      },
   { "db.i",     S(EVAL),0,    1,    "i",    "i",     db                     },
   { "db.k",     S(EVAL),0,    2,    "k",    "k",     NULL, db               },
   { "db.a",     S(EVAL),0,    2,    "a",    "a",     NULL, dba        },
   { "midichn",  S(MIDICHN),0, 1,    "i",    "",      midichn, NULL, NULL    },
   { "pgmassign",S(PGMASSIGN),0, 1,  "",     "iio",   pgmassign, NULL, NULL  },
-    { "pgmassign.S",S(PGMASSIGN),0, 1,  "",     "iSo",   pgmassign_S, NULL, NULL  },
+  { "pgmassign.S",S(PGMASSIGN),0, 1, "",    "iSo",   pgmassign_S, NULL, NULL  },
   { "midiin",   S(MIDIIN),0,  3,    "kkkk", "",      midiin_set, midiin, NULL },
   { "pgmchn",   S(PGMIN),0,  3,     "kk",   "o",     pgmin_set, pgmin, NULL },
   { "ctlchn",   S(CTLIN),0,  3,     "kkk",  "oo",    ctlin_set, ctlin, NULL },
@@ -874,7 +875,7 @@ OENTRY opcodlst_1[] = {
     (SUBR) midiTempoOpcode, (SUBR) midiTempoOpcode, NULL    },
   { "midifilestatus", S(MIDITEMPO),0, 2, "k",    "",
    NULL, (SUBR) midiFileStatus, NULL },
-  { "midinoteoff", S(MIDINOTEON),0,3,"",    "xx",   midinoteoff, midinoteoff, },
+  { "midinoteoff", S(MIDINOTEON),0,3   ,"", "xx",   midinoteoff, midinoteoff, },
   { "midinoteonkey", S(MIDINOTEON),0,3, "", "xx",   midinoteonkey, midinoteonkey },
   { "midinoteoncps", S(MIDINOTEON),0, 3, "", "xx",  midinoteoncps,midinoteoncps },
   { "midinoteonoct", S(MIDINOTEON),0, 3, "", "xx",  midinoteonoct,midinoteonoct },
@@ -890,25 +891,25 @@ OENTRY opcodlst_1[] = {
   { "midipitchbend", S(MIDIPITCHBEND),0,3, "", "xoh",
     midipitchbend, midipitchbend },
   { "mididefault", S(MIDIDEFAULT),0, 3, "", "xx",   mididefault, mididefault },
-  { "invalue",   0xFFFF,   _CR,    0,   NULL,   NULL, NULL, NULL },
-  { "invalue.k", S(INVAL),0, 3, "k", "i",   (SUBR) invalset,(SUBR)  kinval, NULL },
-  { "invalue.i", S(INVAL),0, 1, "i", "i",   (SUBR) invalsetgo, NULL, NULL },
-  { "invalue.kS", S(INVAL),0, 3, "k", "S", (SUBR) invalset_S,(SUBR)  kinval, NULL },
-  { "invalue.iS", S(INVAL),0, 1, "i", "S", (SUBR) invalsetSgo, NULL, NULL },
-  { "invalue.S", S(INVAL),0, 3, "S", "i",
-                                      (SUBR) invalset_string, (SUBR) kinvalS, NULL },
-  { "invalue.SS", S(INVAL),0, 3, "S", "S",
-                                    (SUBR) invalset_string_S, (SUBR) kinvalS, NULL },
-  { "outvalue", S(OUTVAL), _CW, 3, "", "ik", (SUBR) outvalset, (SUBR) koutval, NULL},
+  //{ "invalue",   0xFFFF,   _CR,    0,   NULL,   NULL, NULL, NULL },
+  { "invalue.k", S(INVAL),_CR, 3, "k", "i", (SUBR) invalset,(SUBR)kinval, NULL },
+  { "invalue.i", S(INVAL),_CR, 1, "i", "i", (SUBR) invalsetgo, NULL, NULL },
+  { "invalue.iS", S(INVAL),_CR, 1, "i", "S", (SUBR) invalsetSgo, NULL, NULL },
+  { "invalue.kS", S(INVAL),_CR, 3, "k", "S",(SUBR) invalset_S,(SUBR)kinval, NULL },
+  { "invalue.S", S(INVAL),_CR, 3, "S", "i",
+                                      (SUBR) invalset_string, (SUBR)kinvalS, NULL },
+  { "invalue.SS", S(INVAL),_CR, 3, "S", "S",
+                                    (SUBR) invalset_string_S, (SUBR)kinvalS, NULL },
+  { "outvalue", S(OUTVAL), _CW, 3, "", "ik", (SUBR) outvalset, (SUBR)koutval, NULL},
   { "outvalue.i", S(OUTVAL), _CW, 1, "", "ii", (SUBR) outvalsetgo, NULL, NULL},
-  { "outvalue.k", S(OUTVAL), _CW, 3, "", "Sk",
-                                          (SUBR) outvalset_S, (SUBR) koutval, NULL},
   { "outvalue.Si", S(OUTVAL), _CW, 1, "", "Si",
                                           (SUBR) outvalsetSgo, NULL, NULL},
+  { "outvalue.k", S(OUTVAL), _CW, 3, "", "Sk",
+                                          (SUBR) outvalset_S, (SUBR)koutval, NULL},
   { "outvalue.S", S(OUTVAL), _CW, 3, "", "iS",
-                                     (SUBR) outvalset_string, (SUBR) koutvalS, NULL},
+                                     (SUBR) outvalset_string, (SUBR)koutvalS, NULL},
   { "outvalue.SS", S(OUTVAL), _CW, 3, "", "SS",
-                                   (SUBR) outvalset_string_S, (SUBR) koutvalS, NULL},
+                                   (SUBR) outvalset_string_S, (SUBR)koutvalS, NULL},
   /* IV - Oct 20 2002 */
   { "subinstr", S(SUBINST),0, 3, "mmmmmmmm", "SN",  subinstrset_S, subinstr },
   { "subinstrinit", S(SUBINST),0, 1, "",    "SN",   subinstrset_S, NULL, NULL     },
@@ -916,12 +917,14 @@ OENTRY opcodlst_1[] = {
   { "subinstrinit.i", S(SUBINST),0, 1, "",    "iN",   subinstrset, NULL, NULL     },
   { "nstrnum", S(NSTRNUM),0, 1,     "i",    "S",    nstrnumset_S, NULL, NULL      },
   { "nstrnum.i", S(NSTRNUM),0, 1,     "i",    "i",    nstrnumset, NULL, NULL      },
-  { "turnoff2",   0xFFFB,   _CW,    0, NULL,   NULL,   NULL, NULL, NULL          },
-  { "turnoff2.S",S(TURNOFF2),0,2,     "",     "Skk",  NULL, turnoff2S, NULL     },
-  { "turnoff2.c",S(TURNOFF2),0,2,     "",     "ikk",  NULL, turnoff2k, NULL     },
-  { "turnoff2.k",S(TURNOFF2),0,2,     "",     "kkk",  NULL, turnoff2k, NULL     },
-  { "turnoff2.i",S(TURNOFF2),0,2,     "",     "ikk",  NULL, turnoff2k, NULL     },
-  { "turnoff2.r",S(TURNOFF2),0,2,     "",     "ikk",  NULL, turnoff2k, NULL     },
+  { "nstrstr", S(NSTRSTR),0, 1,       "S",    "i",    nstrstr, NULL, NULL      },
+  { "nstrstr.k", S(NSTRSTR),0, 2,     "S",    "k",    NULL, nstrstr, NULL      },
+  //{ "turnoff2",   0xFFFB,   _CW,    0, NULL,   NULL,   NULL, NULL, NULL          },
+  { "turnoff2.S",S(TURNOFF2),_CW,2,     "",     "Skk",  NULL, turnoff2S, NULL     },
+  { "turnoff2.c",S(TURNOFF2),_CW,2,     "",     "ikk",  NULL, turnoff2k, NULL     },
+  { "turnoff2.k",S(TURNOFF2),_CW,2,     "",     "kkk",  NULL, turnoff2k, NULL     },
+  { "turnoff2.i",S(TURNOFF2),_CW,2,     "",     "ikk",  NULL, turnoff2k, NULL     },
+  { "turnoff2.r",S(TURNOFF2),_CW,2,     "",     "ikk",  NULL, turnoff2k, NULL     },
   { "cngoto", S(CGOTO),0,   3,      "",     "Bl",   ingoto, kngoto, NULL     },
   { "cnkgoto", S(CGOTO),0,   2,      "",     "Bl",   NULL,  kngoto, NULL     },
   { "cingoto", S(CGOTO),0,   1,      "",     "Bl",   ingoto, NULL, NULL     },
@@ -998,9 +1001,9 @@ OENTRY opcodlst_1[] = {
      (SUBR) sprintf_opcode, NULL, NULL                     },
   {  "sprintfk", S(SPRINTF_OP),WR,  3,  "S",    "SUN",
      (SUBR) sprintf_opcode, (SUBR) sprintf_opcode, NULL           },
-  {  "printf_i", S(PRINTF_OP),0,   1,  "",     "SiTN",
+  {  "printf_i", S(PRINTF_OP),0,   1,  "",     "SiN", /* SiTN */
      (SUBR) printf_opcode_init, NULL, NULL                 },
-  {  "printf",   S(PRINTF_OP),WR,   3,  "",     "SkUN",
+  {  "printf",   S(PRINTF_OP),WR,   3,  "",     "SkN", /* SkUN */
      (SUBR) printf_opcode_set, (SUBR) printf_opcode_perf, NULL    },
   {  "puts",     S(PUTS_OP),WR,     3,  "",     "Sko",
      (SUBR) puts_opcode_init, (SUBR) puts_opcode_perf, NULL       },
@@ -1051,12 +1054,14 @@ OENTRY opcodlst_1[] = {
 #ifdef HAVE_CURL
   {  "strfromurl", S(STRCPY_OP), 0, 1, "S", "S", (SUBR) str_from_url     },
 #endif
-  {  "changed.S", S(STRINDEX_OP),0, 3, "k",   "S",
+  {  "changed.S", S(STRCHGD),_QQ, 3, "k",   "S",
      (SUBR) str_changed, (SUBR) str_changed_k, NULL       },
-  { "loop_lt",   0xfffb                                                  },
-  { "loop_le",   0xfffb                                                  },
-  { "loop_gt",   0xfffb                                                  },
-  { "loop_ge",   0xfffb                                                  },
+  {  "changed2.S", S(STRCHGD),0, 3, "k",   "S",
+     (SUBR) str_changed, (SUBR) str_changed_k, NULL       },
+  /* { "loop_lt",   0xfffb                                                  }, */
+  /* { "loop_le",   0xfffb                                                  }, */
+  /* { "loop_gt",   0xfffb                                                  }, */
+  /* { "loop_ge",   0xfffb                                                  }, */
   { "loop_lt.i", S(LOOP_OPS),0,  1,  "", "iiil", (SUBR) loop_l_i, NULL, NULL   },
   { "loop_le.i", S(LOOP_OPS),0,  1,  "", "iiil", (SUBR) loop_le_i, NULL, NULL  },
   { "loop_gt.i", S(LOOP_OPS),0,  1,  "", "iiil", (SUBR) loop_g_i, NULL, NULL   },
@@ -1066,17 +1071,35 @@ OENTRY opcodlst_1[] = {
   { "loop_gt.k", S(LOOP_OPS),0,  2,  "", "kkkl", NULL, (SUBR) loop_g_p, NULL   },
   { "loop_ge.k", S(LOOP_OPS),0,  2,  "", "kkkl", NULL, (SUBR) loop_ge_p, NULL  },
   { "chnget",      0xFFFF,    _CR                                             },
-  { "chnget.i",    S(CHNGET),0,           1,      "i",            "S",
+  { "chnget.i",    S(CHNGET),_CR,           1,      "i",            "S",
     (SUBR) chnget_opcode_init_i, NULL, NULL               },
-  { "chnget.k",    S(CHNGET),0,           3,      "k",            "S",
+  { "chnget.k",    S(CHNGET),_CR,           3,      "k",            "S",
     (SUBR) chnget_opcode_init_k, (SUBR) notinit_opcode_stub, NULL },
-  { "chnget.a",    S(CHNGET),0,           3,      "a",            "S",
+  { "chngeti.i",    S(CHNGET),_CR,           1,      "i[]",            "S[]",
+    (SUBR) chnget_array_opcode_init_i, NULL, NULL               },
+  { "chngeta.a",    S(CHNGET),_CR,           3,      "a[]",            "S[]",
+    (SUBR) chnget_array_opcode_init, (SUBR) notinit_opcode_stub, NULL },
+  { "chngets.s",    S(CHNGET),_CR,           3,      "S[]",            "S[]",
+    (SUBR) chnget_array_opcode_init, (SUBR) notinit_opcode_stub, NULL },
+  { "chngetk.k",    S(CHNGET),_CR,           3,      "k[]",            "S[]",
+    (SUBR) chnget_array_opcode_init, (SUBR) notinit_opcode_stub, NULL },
+  { "chnget.a",    S(CHNGET),_CR,           3,      "a",            "S",
     (SUBR) chnget_opcode_init_a, (SUBR) notinit_opcode_stub },
-  { "chnget.S",    S(CHNGET),0,           3,      "S",            "S",
+  { "chnget.S",    S(CHNGET),_CR,           3,      "S",            "S",
     (SUBR) chnget_opcode_init_S, (SUBR) chnget_opcode_perf_S, NULL},
-  { "chngetks",    S(CHNGET),0,           2,      "S",            "S",
+  { "chngetks",    S(CHNGET),_CR,           2,      "S",            "S",
     NULL, (SUBR) chnget_opcode_perf_S, NULL},
-  { "chnset",      0xFFFB,              _CW                               },
+  //{ "chnset",      0xFFFB,              _CW                               },
+
+  { "chnseti.i",    S(CHNGET),_CW,          1,      "",             "i[]S[]",
+    (SUBR) chnset_array_opcode_init_i, NULL, NULL               },
+  { "chnsetk.k",    S(CHNGET),_CW,           3,      "",             "k[]S[]",
+    (SUBR) chnset_array_opcode_init, (SUBR) notinit_opcode_stub, NULL },
+  { "chnseta.a",    S(CHNGET),_CW,           3,      "",             "a[]S[]",
+    (SUBR) chnset_array_opcode_init, (SUBR) notinit_opcode_stub, NULL },
+  { "chnsets.s",    S(CHNGET),_CW,           3,      "",             "S[]S[]",
+    (SUBR) chnset_array_opcode_init, (SUBR) notinit_opcode_stub, NULL },
+
   { "chnset.i",    S(CHNGET),_CW,          1,      "",             "iS",
     (SUBR) chnset_opcode_init_i, NULL, NULL               },
   //  { "chnset.r",    S(CHNGET),0,           1,      "",             "iS",
@@ -1093,15 +1116,17 @@ OENTRY opcodlst_1[] = {
     NULL, (SUBR) chnset_opcode_perf_S, NULL },
   { "chnmix",      S(CHNGET),           _CB, 3,      "",             "aS",
     (SUBR) chnmix_opcode_init, (SUBR) notinit_opcode_stub  },
-  { "chnclear",    S(CHNCLEAR),        _CW, 3,      "",             "S",
+  { "chnclear",    S(CHNCLEAR),        _CW, 3,      "",             "W",
     (SUBR) chnclear_opcode_init, (SUBR) notinit_opcode_stub },
   { "chn_k",       S(CHN_OPCODE_K),    _CW, 1,      "",             "SiooooooooN",
     (SUBR) chn_k_opcode_init, NULL, NULL                  },
+  { "chn_k",       S(CHN_OPCODE_K),    _CW, 1,      "",             "SSooooooooN",
+    (SUBR) chn_k_opcode_init_S, NULL, NULL},
   { "chn_a",       S(CHN_OPCODE),      _CW, 1,      "",             "Si",
     (SUBR) chn_a_opcode_init, NULL, NULL                  },
   { "chn_S",       S(CHN_OPCODE),      _CW, 1,      "",             "Si",
     (SUBR) chn_S_opcode_init, NULL, NULL                  },
-  { "chnexport",   0xFFFF,  0,    0,   NULL,   NULL, NULL, NULL },
+  //{ "chnexport",   0xFFFF,  0,    0,   NULL,   NULL, NULL, NULL },
   { "chnexport.i", S(CHNEXPORT_OPCODE),0, 1,      "i",            "Sioooo",
     (SUBR) chnexport_opcode_init, NULL, NULL              },
   { "chnexport.k", S(CHNEXPORT_OPCODE),0, 1,      "k",            "Sioooo",
@@ -1118,17 +1143,17 @@ OENTRY opcodlst_1[] = {
       { "chnsend",     S(CHNSEND),0,          3,      "",             "So",
       (SUBR) chnsend_opcode_init, (SUBR) notinit_opcode_stub, NULL },
   */
-  { "chano",       0xFFFD,  _CW, 0,      NULL, NULL, NULL, NULL },
-  { "chano.k",     S(ASSIGN),0,           2,      "",             "kk",
+  //{ "chano",       0xFFFD,  _CW, 0,      NULL, NULL, NULL, NULL },
+  { "chano.k",     S(ASSIGN),_CW,           2,      "",             "kk",
     NULL, (SUBR) chano_opcode_perf_k, NULL                },
-  { "chano.a",     S(ASSIGN),0,           2,      "",             "ak",
+  { "chano.a",     S(ASSIGN),_CW,           2,      "",             "ak",
     NULL, (SUBR) chano_opcode_perf_a                },
   { "pvsout",     S(FCHAN),0,           3,      "",             "fk",
     (SUBR) pvsout_init, (SUBR) pvsout_perf, NULL                        },
-  { "chani",      0xFFFF,  _CR,      0,   NULL, NULL, NULL, NULL },
-  { "chani.k",     S(ASSIGN),0,           2,      "k",            "k",
+  //{ "chani",      0xFFFF,  _CR,      0,   NULL, NULL, NULL, NULL },
+  { "chani.k",     S(ASSIGN),_CR,           2,      "k",            "k",
     NULL, (SUBR) chani_opcode_perf_k, NULL                },
-  { "chani.a",     S(ASSIGN),0,           2,      "a",            "k",
+  { "chani.a",     S(ASSIGN),_CR,           2,      "a",            "k",
     NULL, (SUBR) chani_opcode_perf_a                },
   { "pvsin",     S(FCHAN),0,           3,      "f",            "kooopo",
     (SUBR)  pvsin_init, (SUBR) pvsin_perf, NULL                  },
@@ -1162,14 +1187,34 @@ OENTRY opcodlst_1[] = {
   { "readscore",  S(COMPILE), 0, 1, "i", "S",  (SUBR) read_score_i, NULL, NULL },
   { "return",  S(RETVAL), 0, 1, "", "i",  (SUBR) retval_i, NULL, NULL },
   /* ----------------------------------------------------------------------- */
-  { "monitor",  sizeof(MONITOR_OPCODE), 0, 3,  "mmmmmmmmmmmmmmmmmmmmmmmm", "",
+  { "monitor",  sizeof(MONITOR_OPCODE), IB, 3,  "mmmmmmmmmmmmmmmmmmmmmmmm", "",
     (SUBR) monitor_opcode_init, (SUBR) notinit_opcode_stub,  NULL },
-  { "outrg", S(OUTRANGE), 0,3, "", "ky",
+  { "outrg", S(OUTRANGE), IR,3, "", "ky",
     (SUBR)outRange_i, (SUBR)outRange},
   { "nchnls_hw", S(ASSIGN), 0,1, "ii", "",
     (SUBR)hw_channels},
    { "midiarp",   S(MIDIARP),0,  3,    "kk", "kO",
      midiarp_set, midiarp, NULL },
+   {"lpcfilter", S(LPCFIL), 0, 3, "a", "akkiiio",
+   (SUBR) lpfil_init, (SUBR) lpfil_perf},
+   {"lpcfilter", S(LPCFIL2), 0, 3, "a", "aakkiio",
+   (SUBR) lpfil2_init, (SUBR) lpfil2_perf},
+   {"allpole", S(LPCFIL3), 0, 3, "a", "ak[]",
+   (SUBR) lpfil3_init, (SUBR) lpfil3_perf},
+   {"lpcanal", S(LPREDA), 0, 3, "k[]kkk", "kkiiio",
+   (SUBR) lpred_alloc, (SUBR) lpred_run},
+   {"lpcanal", S(LPREDA2), 0, 3, "k[]kkk", "akkiio",
+   (SUBR) lpred_alloc2, (SUBR) lpred_run2},
+   {"lpcanal", S(LPREDA), 0, 1, "i[]iii", "iiii",
+   (SUBR) lpred_i, NULL},
+   {"pvslpc", S(LPCPVS), 0, 3, "f", "aiiio",
+    (SUBR) lpcpvs_init, (SUBR) lpcpvs},
+   {"pvscfs", S(PVSCFS), 0, 3, "k[]kk", "fip",
+    (SUBR) pvscoefs_init, (SUBR) pvscoefs},
+   {"apoleparams", S(CF2P), 0, 3, "k[]", "k[]",
+    (SUBR) coef2parm_init, (SUBR) coef2parm},
+   {"resonbnk", S(RESONB), 0, 3, "a", "ak[]kkipoo",
+   (SUBR) resonbnk_init, (SUBR) resonbnk},
   /* terminate list */
   {  NULL, 0, 0, 0, NULL, NULL, NULL, NULL, NULL       }
 };
