@@ -109,12 +109,14 @@ static int init_pass(CSOUND *csound, INSDS *ip) {
     csoundLockMutex(csound->init_pass_threadlock);
   csound->curip = ip;
   csound->ids = (OPDS *)ip;
-  while (error == 0 && (csound->ids = csound->ids->nxti) != NULL){
+  csound->mode = 1;
+  while (error == 0 && (csound->ids = csound->ids->nxti) != NULL) {
+    csound->op = csound->ids->optext->t.oentry->opname;
     if (UNLIKELY(csound->oparms->odebug))
-      csound->Message(csound, "init %s:\n",
-                      csound->ids->optext->t.oentry->opname);
+      csound->Message(csound, "init %s:\n", csound->op);
     error = (*csound->ids->iopadr)(csound, csound->ids);
   }
+  csound->mode = 0;
   if(csound->oparms->realtime)
     csoundUnlockMutex(csound->init_pass_threadlock);
   return error;
@@ -129,13 +131,15 @@ static int reinit_pass(CSOUND *csound, INSDS *ip, OPDS *ids) {
   }
   csound->curip = ip;
   csound->ids = ids;
+  csound->mode = 1;
   while (error == 0 && (csound->ids = csound->ids->nxti) != NULL &&
          (csound->ids->iopadr != (SUBR) rireturn)){
-   if (UNLIKELY(csound->oparms->odebug))
-      csound->Message(csound, "reinit %s:\n",
-                      csound->ids->optext->t.oentry->opname);
+    csound->op = csound->ids->optext->t.oentry->opname;
+    if (UNLIKELY(csound->oparms->odebug))
+      csound->Message(csound, "reinit %s:\n", csound->op);
     error = (*csound->ids->iopadr)(csound, csound->ids);
   }
+  csound->mode = 0;
 
   ATOMIC_SET8(ip->actflg, 1);
   csound->reinitflag = ip->reinitflag = 0;
@@ -245,9 +249,12 @@ int init0(CSOUND *csound)
   ip->onedkr = csound->onedkr;
   ip->kicvt = csound->kicvt;
   csound->inerrcnt = 0;
+  csound->mode = 1;
   while ((csound->ids = csound->ids->nxti) != NULL) {
+    csound->op = csound->ids->optext->t.oentry->opname;
     (*csound->ids->iopadr)(csound, csound->ids);  /*   run all i-code     */
   }
+  csound->mode = 0;
   return csound->inerrcnt;                        /*   return errcnt      */
 }
 
@@ -400,6 +407,7 @@ int insert_event(CSOUND *csound, int insno, EVTBLK *newevtp)
     ip->nxtact = nxtp;
     ip->prvact = prvp;
     prvp->nxtact = ip;
+    ip->tieflag = 0;
     ip->actflg++;                   /*    and mark the instr active */
   }
 
@@ -485,11 +493,16 @@ int insert_event(CSOUND *csound, int insno, EVTBLK *newevtp)
     start_time_kcycles = start_time_samps/csound->ksmps;
     ip->ksmps_offset = start_time_samps - start_time_kcycles*csound->ksmps;
     /* with no p3 or xtratim values, can't set the sample accur duration */
-    if (ip->p3.value > 0 && ip->xtratim == 0)
-      ip->no_end = csound->ksmps -
-        ((int)duration_samps+ip->ksmps_offset)%csound->ksmps;
+    if (ip->p3.value > 0 && ip->xtratim == 0 ){
+      int tmp = ((int)duration_samps+ip->ksmps_offset)%csound->ksmps;
+      if (tmp != 0)ip->no_end = csound->ksmps - tmp; else ip->no_end = 0;
+      //ip->no_end = (csound->ksmps -
+      //              ((int)duration_samps+ip->ksmps_offset)%csound->ksmps)%csound->ksmps;
     /* the ksmps_no_end field is initially 0, set to no_end in the last
        perf cycle */
+    //  printf("*** duration_samps %d ip->ksmps_offset %d csound->ksmps %d ==> %d\n",
+    //         (int)duration_samps, ip->ksmps_offset, csound->ksmps, ip->no_end);
+    }
     else ip->no_end = 0;
     ip->ksmps_no_end = 0;
   }
@@ -768,8 +781,8 @@ int insert_midi(CSOUND *csound, int insno, MCHNBLK *chn, MEVENT *mep)
     value = value * csound->e0dbfs;
     pfield->value = value;
     if (UNLIKELY(O->msglevel & WARNMSG)) {
-      csound->Message(csound, "  midiVelocityAmp: pfield: %3d  value: %3d\n",
-                      pfield_index, (int)pfield->value);
+      csound->Message(csound, "  midiVelocityAmp: pfield: %3d  value: %.3f\n",
+                      pfield_index, pfield->value);
     }
     if (pmax < pfield_index) pmax = pfield_index;
   }
@@ -1118,6 +1131,8 @@ void infoff(CSOUND *csound, MYFLT p1)   /* turn off an indef copy of instr p1 */
                   p1);
 }
 
+void do_baktrace(CSOUND *, uint64_t);
+
 int csoundInitError(CSOUND *csound, const char *s, ...)
 {
   va_list args;
@@ -1131,6 +1146,8 @@ int csoundInitError(CSOUND *csound, const char *s, ...)
     va_end(args);
     csound->LongJmp(csound, 1);
   }
+  if (csound->mode != 1)
+    csound->Message(csound, Str("InitError in wrong mode %d\n"), csound->mode);
   /* IV - Oct 16 2002: check for subinstr and user opcode */
   ip = csound->ids->insdshead;
   if (ip->opcod_iobufs) {
@@ -1140,27 +1157,32 @@ int csoundInitError(CSOUND *csound, const char *s, ...)
       ip = ((OPCOD_IOBUFS*) ip->opcod_iobufs)->parent_ip;
     } while (ip->opcod_iobufs);
     if (op)
-      snprintf(buf, 512, Str("INIT ERROR in instr %d (opcode %s): "),
-               ip->insno, op->name);
+      snprintf(buf, 512, Str("INIT ERROR in instr %d (opcode %s) line %d: "),
+               ip->insno, op->name, csound->ids->optext->t.linenum);
     else
-      snprintf(buf, 512, Str("INIT ERROR in instr %d (subinstr %d): "),
-               ip->insno, csound->ids->insdshead->insno);
+      snprintf(buf, 512, Str("INIT ERROR in instr %d (subinstr %d) line %d: "),
+               ip->insno, csound->ids->insdshead->insno,
+               csound->ids->optext->t.linenum);
   }
   else
-    snprintf(buf, 512, Str("INIT ERROR in instr %d: "), ip->insno);
+    snprintf(buf, 512, Str("INIT ERROR in instr %d (opcode %s) line %d: "),
+             ip->insno, csound->op, csound->ids->optext->t.linenum);
   va_start(args, s);
   csoundErrMsgV(csound, buf, s, args);
   va_end(args);
+  do_baktrace(csound, csound->ids->optext->t.locn);
   putop(csound, &(csound->ids->optext->t));
-
   return ++(csound->inerrcnt);
 }
 
-int csoundPerfError(CSOUND *csound, INSDS *ip, const char *s, ...)
+int csoundPerfError(CSOUND *csound, OPDS *h, const char *s, ...)
 {
   va_list args;
   char    buf[512];
-
+  INSDS *ip = h->insdshead;
+  TEXT t = h->optext->t;
+  if (csound->mode != 2)
+    csound->Message(csound, Str("PerfError in wrong mode %d\n"), csound->mode);
   if (ip->opcod_iobufs) {
     OPCODINFO *op = ((OPCOD_IOBUFS*) ip->opcod_iobufs)->opcode_info;
     /* find top level instrument instance */
@@ -1168,17 +1190,19 @@ int csoundPerfError(CSOUND *csound, INSDS *ip, const char *s, ...)
       ip = ((OPCOD_IOBUFS*) ip->opcod_iobufs)->parent_ip;
     } while (ip->opcod_iobufs);
     if (op)
-      snprintf(buf, 512, Str("PERF ERROR in instr %d (opcode %s): "),
-               ip->insno, op->name);
+      snprintf(buf, 512, Str("PERF ERROR in instr %d (opcode %s) line %d: "),
+               ip->insno, op->name, t.linenum);
     else
-      snprintf(buf, 512, Str("PERF ERROR in instr %d (subinstr %d): "),
-               ip->insno, ip->insno);
+      snprintf(buf, 512, Str("PERF ERROR in instr %d (subinstr %d) line %d: "),
+               ip->insno, ip->insno, t.linenum);
   }
   else
-    snprintf(buf, 512, Str("PERF ERROR in instr %d: "), ip->insno);
+    snprintf(buf, 512, Str("PERF ERROR in instr %d (opcode %s) line %d: "),
+             ip->insno, csound->op, t.linenum);
   va_start(args, s);
   csoundErrMsgV(csound, buf, s, args);
   va_end(args);
+  do_baktrace(csound, t.locn);
   if (ip->pds)
     putop(csound, &(ip->pds->optext->t));
   csoundMessage(csound, Str("   note aborted\n"));
@@ -1204,7 +1228,7 @@ int subinstrset_(CSOUND *csound, SUBINST *p, int instno)
                                        "args greater than nchnls"));
   }
   /* IV - Oct 9 2002: copied this code from useropcdset() to fix some bugs */
-  if (!(pip->reinitflag | pip->tieflag)) {
+  if (!(pip->reinitflag | pip->tieflag) || p->ip == NULL) {
     /* get instance */
     if (csound->engineState.instrtxtp[instno]->act_instance == NULL)
       instance(csound, instno);
@@ -1289,11 +1313,15 @@ int subinstrset_(CSOUND *csound, SUBINST *p, int instno)
                    (int32) csound->nspout * sizeof(MYFLT), &p->saved_spout);
 
   /* do init pass for this instr */
+  csound->curip = p->ip;        /* **** NEW *** */
   p->ip->init_done = 0;
   csound->ids = (OPDS *)p->ip;
+  csound->mode = 1;
   while ((csound->ids = csound->ids->nxti) != NULL) {
+    csound->op = csound->ids->optext->t.oentry->opname;
     (*csound->ids->iopadr)(csound, csound->ids);
   }
+  csound->mode = 0;
   p->ip->init_done = 1;
   /* copy length related parameters back to caller instr */
   saved_curip->xtratim = csound->curip->xtratim;
@@ -1314,6 +1342,7 @@ int subinstrset_S(CSOUND *csound, SUBINST *p){
   init_op = (p->h.opadr == NULL ? 1 : 0);
   inarg_ofs = (init_op ? 0 : SUBINSTNUMOUTS);
   instno = strarg2insno(csound, ((STRINGDAT *)p->ar[inarg_ofs])->data, 1);
+  if (UNLIKELY(instno==NOT_AN_INSTRUMENT)) instno = -1;
   return subinstrset_(csound,p,instno);
 }
 
@@ -1356,177 +1385,184 @@ int useropcd1(CSOUND *, UOPCODE*), useropcd2(CSOUND *, UOPCODE*);
 
 int useropcdset(CSOUND *csound, UOPCODE *p)
 {
-  OPDS         *saved_ids = csound->ids;
-  INSDS        *parent_ip = csound->curip, *lcurip;
-  INSTRTXT     *tp;
-  unsigned int instno;
-  unsigned int pcnt;
-  unsigned int i, n;
-  OPCODINFO    *inm;
-  OPCOD_IOBUFS *buf = NULL;
-  MYFLT ksmps_scale;
-  unsigned int local_ksmps;
-  /* default ksmps */
-  local_ksmps = CS_KSMPS;
-  ksmps_scale = 1;
-  /* look up the 'fake' instr number, and opcode name */
-  inm = (OPCODINFO*) p->h.optext->t.oentry->useropinfo;
-  instno = inm->instno;
-  tp = csound->engineState.instrtxtp[instno];
-  if (tp == NULL)
-    return csound->InitError(csound, "Can't find instr %d (UDO %s)\n",
-                             instno, inm->name);
-  /* set local ksmps if defined by user */
-  n = p->OUTOCOUNT + p->INCOUNT - 1;
-
-  if (*(p->ar[n]) != FL(0.0)) {
-    i = (unsigned int) *(p->ar[n]);
-    if (UNLIKELY(i < 1 || i > csound->ksmps ||
-                 ((CS_KSMPS / i) * i) != CS_KSMPS)) {
-      return csoundInitError(csound, Str("%s: invalid local ksmps value: %d"),
-                             inm->name, i);
-    }
-    local_ksmps = i;
-  }
-
-  if (!p->ip) {
-
-    /* search for already allocated, but not active instance */
-    /* if none was found, allocate a new instance */
+    OPDS         *saved_ids = csound->ids;
+    INSDS        *parent_ip = csound->curip, *lcurip;
+    INSTRTXT     *tp;
+    unsigned int instno;
+    unsigned int pcnt;
+    unsigned int i, n;
+    OPCODINFO    *inm;
+    OPCOD_IOBUFS *buf = NULL;
+    MYFLT ksmps_scale;
+    unsigned int local_ksmps;
+    /* default ksmps */
+    local_ksmps = CS_KSMPS;
+    ksmps_scale = 1;
+    /* look up the 'fake' instr number, and opcode name */
+    inm = (OPCODINFO*) p->h.optext->t.oentry->useropinfo;
+    instno = inm->instno;
     tp = csound->engineState.instrtxtp[instno];
-    if (tp == NULL) {
-      return csound->InitError(csound, "Can't find instr %d (UDO %s)\n",
+    if (tp == NULL)
+      return csound->InitError(csound, Str("Cannot find instr %d (UDO %s)\n"),
                                instno, inm->name);
-    }
-    if (!tp->act_instance)
-      instance(csound, instno);
-    lcurip = tp->act_instance;            /* use free intance, and  */
-    tp->act_instance = lcurip->nxtact;    /* remove from chain      */
-    lcurip->actflg++;                     /*    and mark the instr active */
-    tp->active++;
-    tp->instcnt++;
-    /* link into deact chain */
-    lcurip->opcod_deact = parent_ip->opcod_deact;
-    lcurip->subins_deact = NULL;
-    parent_ip->opcod_deact = (void*) p;
-    p->ip = lcurip;
-    /* IV - Nov 10 2002: set up pointers to I/O buffers */
-    buf = p->buf = (OPCOD_IOBUFS*) lcurip->opcod_iobufs;
-    buf->opcode_info = inm;
-    /* initialise perf time address lists */
-    buf->iobufp_ptrs[0] = buf->iobufp_ptrs[1] = NULL;
-    buf->iobufp_ptrs[2] = buf->iobufp_ptrs[3] = NULL;
-    buf->iobufp_ptrs[4] = buf->iobufp_ptrs[5] = NULL;
-    buf->iobufp_ptrs[6] = buf->iobufp_ptrs[7] = NULL;
-    buf->iobufp_ptrs[8] = buf->iobufp_ptrs[9] = NULL;
-    buf->iobufp_ptrs[10] = buf->iobufp_ptrs[11] = NULL;
-    /* store parameters of input and output channels, and parent ip */
-    buf->uopcode_struct = (void*) p;
-    buf->parent_ip = p->parent_ip = parent_ip;
-  }
+    /* set local ksmps if defined by user */
+    n = p->OUTOCOUNT + p->INCOUNT - 1;
 
-  /* copy parameters from the caller instrument into our subinstrument */
-  lcurip = p->ip;
-
-  /* set the local ksmps values */
-  if (local_ksmps != CS_KSMPS) {
-    /* this is the case when p->ip->ksmps != p->h.insdshead->ksmps */
-    lcurip->ksmps = local_ksmps;
-    ksmps_scale = CS_KSMPS / local_ksmps;
-    lcurip->onedksmps =  FL(1.0) / (MYFLT) local_ksmps;
-    lcurip->ekr = csound->esr / (MYFLT) local_ksmps;
-    lcurip->onedkr = FL(1.0) / lcurip->ekr;
-    lcurip->kicvt = (MYFLT) FMAXLEN /lcurip->ekr;
-    lcurip->kcounter *= ksmps_scale;
-  } else {
-    lcurip->ksmps = CS_KSMPS;
-    lcurip->kcounter = CS_KCNT;
-    lcurip->ekr = CS_EKR;
-    lcurip->onedkr = CS_ONEDKR;
-    lcurip->onedksmps = CS_ONEDKSMPS;
-    lcurip->kicvt = CS_KICVT;
-  }
-
-  /* VL 13-12-13 */
-  /* this sets ksmps and kr local variables */
-  /* create local ksmps variable and init with ksmps */
-  if (lcurip->lclbas != NULL) {
-    CS_VARIABLE *var =
-      csoundFindVariableWithName(csound, lcurip->instr->varPool, "ksmps");
-    *((MYFLT *)(var->memBlockIndex + lcurip->lclbas)) = lcurip->ksmps;
-    /* same for kr */
-    var =
-      csoundFindVariableWithName(csound, lcurip->instr->varPool, "kr");
-    *((MYFLT *)(var->memBlockIndex + lcurip->lclbas)) = lcurip->ekr;
-  }
-
-  lcurip->m_chnbp = parent_ip->m_chnbp;       /* MIDI parameters */
-  lcurip->m_pitch = parent_ip->m_pitch;
-  lcurip->m_veloc = parent_ip->m_veloc;
-  lcurip->xtratim = parent_ip->xtratim * ksmps_scale;
-  lcurip->m_sust = 0;
-  lcurip->relesing = parent_ip->relesing;
-  lcurip->offbet = parent_ip->offbet;
-  lcurip->offtim = parent_ip->offtim;
-  lcurip->nxtolap = NULL;
-  lcurip->ksmps_offset = parent_ip->ksmps_offset;
-  lcurip->ksmps_no_end = parent_ip->ksmps_no_end;
-  lcurip->tieflag = parent_ip->tieflag;
-  lcurip->reinitflag = parent_ip->reinitflag;
-  /* copy all p-fields, including p1 (will this work ?) */
-  if (tp->pmax > 3) {         /* requested number of p-fields */
-    n = tp->pmax; pcnt = 0;
-    while (pcnt < n) {
-      if ((i = csound->engineState.instrtxtp[parent_ip->insno]->pmax) > pcnt) {
-        if (i > n) i = n;
-        /* copy next block of p-fields */
-        memcpy(&(lcurip->p1) + pcnt, &(parent_ip->p1) + pcnt,
-               (size_t) ((i - pcnt) * sizeof(CS_VAR_MEM)));
-        pcnt = i;
+    if (*(p->ar[n]) != FL(0.0)) {
+      i = (unsigned int) *(p->ar[n]);
+      if (UNLIKELY(i < 1 || i > csound->ksmps ||
+                   ((CS_KSMPS / i) * i) != CS_KSMPS)) {
+        return csoundInitError(csound, Str("%s: invalid local ksmps value: %d"),
+                               inm->name, i);
       }
-      /* top level instr reached */
-      if (parent_ip->opcod_iobufs == NULL) break;
-      parent_ip = ((OPCOD_IOBUFS*) parent_ip->opcod_iobufs)->parent_ip;
+      local_ksmps = i;
     }
-  }
-  else
-    memcpy(&(lcurip->p1), &(parent_ip->p1), 3 * sizeof(CS_VAR_MEM));
+
+    if (!p->ip) {
+
+      /* search for already allocated, but not active instance */
+      /* if none was found, allocate a new instance */
+      tp = csound->engineState.instrtxtp[instno];
+      if (tp == NULL) {
+        return csound->InitError(csound, Str("Cannot find instr %d (UDO %s)\n"),
+                                 instno, inm->name);
+      }
+      if (!tp->act_instance)
+        instance(csound, instno);
+      lcurip = tp->act_instance;            /* use free instance, and */
+      tp->act_instance = lcurip->nxtact;    /* remove from chain      */
+      if (lcurip->opcod_iobufs==NULL)
+        return csound->InitError(csound, "Broken redefinition of UDO %d (UDO %s)\n",
+                                 instno, inm->name);
+      lcurip->actflg++;                     /*    and mark the instr active */
+      tp->active++;
+      tp->instcnt++;
+      /* link into deact chain */
+      lcurip->opcod_deact = parent_ip->opcod_deact;
+      lcurip->subins_deact = NULL;
+      parent_ip->opcod_deact = (void*) p;
+      p->ip = lcurip;
+      /* IV - Nov 10 2002: set up pointers to I/O buffers */
+      buf = p->buf = (OPCOD_IOBUFS*) lcurip->opcod_iobufs;
+      buf->opcode_info = inm;
+      /* initialise perf time address lists */
+      /* **** Could be a memset **** */
+      buf->iobufp_ptrs[0] = buf->iobufp_ptrs[1] = NULL;
+      buf->iobufp_ptrs[2] = buf->iobufp_ptrs[3] = NULL;
+      buf->iobufp_ptrs[4] = buf->iobufp_ptrs[5] = NULL;
+      buf->iobufp_ptrs[6] = buf->iobufp_ptrs[7] = NULL;
+      buf->iobufp_ptrs[8] = buf->iobufp_ptrs[9] = NULL;
+      buf->iobufp_ptrs[10] = buf->iobufp_ptrs[11] = NULL;
+      /* store parameters of input and output channels, and parent ip */
+      buf->uopcode_struct = (void*) p;
+      buf->parent_ip = p->parent_ip = parent_ip;
+    }
+
+    /* copy parameters from the caller instrument into our subinstrument */
+    lcurip = p->ip;
+
+    /* set the local ksmps values */
+    if (local_ksmps != CS_KSMPS) {
+      /* this is the case when p->ip->ksmps != p->h.insdshead->ksmps */
+      lcurip->ksmps = local_ksmps;
+      ksmps_scale = CS_KSMPS / local_ksmps;
+      lcurip->onedksmps =  FL(1.0) / (MYFLT) local_ksmps;
+      lcurip->ekr = csound->esr / (MYFLT) local_ksmps;
+      lcurip->onedkr = FL(1.0) / lcurip->ekr;
+      lcurip->kicvt = (MYFLT) FMAXLEN /lcurip->ekr;
+      lcurip->kcounter *= ksmps_scale;
+    } else {
+      lcurip->ksmps = CS_KSMPS;
+      lcurip->kcounter = CS_KCNT;
+      lcurip->ekr = CS_EKR;
+      lcurip->onedkr = CS_ONEDKR;
+      lcurip->onedksmps = CS_ONEDKSMPS;
+      lcurip->kicvt = CS_KICVT;
+    }
+
+    /* VL 13-12-13 */
+    /* this sets ksmps and kr local variables */
+    /* create local ksmps variable and init with ksmps */
+    if (lcurip->lclbas != NULL) {
+      CS_VARIABLE *var =
+        csoundFindVariableWithName(csound, lcurip->instr->varPool, "ksmps");
+      *((MYFLT *)(var->memBlockIndex + lcurip->lclbas)) = lcurip->ksmps;
+      /* same for kr */
+      var =
+        csoundFindVariableWithName(csound, lcurip->instr->varPool, "kr");
+      *((MYFLT *)(var->memBlockIndex + lcurip->lclbas)) = lcurip->ekr;
+    }
+
+    lcurip->m_chnbp = parent_ip->m_chnbp;       /* MIDI parameters */
+    lcurip->m_pitch = parent_ip->m_pitch;
+    lcurip->m_veloc = parent_ip->m_veloc;
+    lcurip->xtratim = parent_ip->xtratim * ksmps_scale;
+    lcurip->m_sust = 0;
+    lcurip->relesing = parent_ip->relesing;
+    lcurip->offbet = parent_ip->offbet;
+    lcurip->offtim = parent_ip->offtim;
+    lcurip->nxtolap = NULL;
+    lcurip->ksmps_offset = parent_ip->ksmps_offset;
+    lcurip->ksmps_no_end = parent_ip->ksmps_no_end;
+    lcurip->tieflag = parent_ip->tieflag;
+    lcurip->reinitflag = parent_ip->reinitflag;
+    /* copy all p-fields, including p1 (will this work ?) */
+    if (tp->pmax > 3) {         /* requested number of p-fields */
+      n = tp->pmax; pcnt = 0;
+      while (pcnt < n) {
+        if ((i = csound->engineState.instrtxtp[parent_ip->insno]->pmax) > pcnt) {
+          if (i > n) i = n;
+          /* copy next block of p-fields */
+          memcpy(&(lcurip->p1) + pcnt, &(parent_ip->p1) + pcnt,
+                 (size_t) ((i - pcnt) * sizeof(CS_VAR_MEM)));
+          pcnt = i;
+        }
+        /* top level instr reached */
+        if (parent_ip->opcod_iobufs == NULL) break;
+        parent_ip = ((OPCOD_IOBUFS*) parent_ip->opcod_iobufs)->parent_ip;
+      }
+    }
+    else
+      memcpy(&(lcurip->p1), &(parent_ip->p1), 3 * sizeof(CS_VAR_MEM));
 
 
-  /* do init pass for this instr */
-  csound->curip = lcurip;
-  csound->ids = (OPDS *) (lcurip->nxti);
-  ATOMIC_SET(p->ip->init_done, 0);
-  while (csound->ids != NULL) {
-    (*csound->ids->iopadr)(csound, csound->ids);
-    csound->ids = csound->ids->nxti;
-  }
-  ATOMIC_SET(p->ip->init_done, 1);
-  /* copy length related parameters back to caller instr */
-  parent_ip->relesing = lcurip->relesing;
-  parent_ip->offbet = lcurip->offbet;
-  parent_ip->offtim = lcurip->offtim;
-  parent_ip->p3 = lcurip->p3;
-  local_ksmps = lcurip->ksmps;
+    /* do init pass for this instr */
+    csound->curip = lcurip;
+    csound->ids = (OPDS *) (lcurip->nxti);
+    ATOMIC_SET(p->ip->init_done, 0);
+    csound->mode = 1;
+    while (csound->ids != NULL) {
+      csound->op = csound->ids->optext->t.oentry->opname;
+      (*csound->ids->iopadr)(csound, csound->ids);
+      csound->ids = csound->ids->nxti;
+    }
+    csound->mode = 0;
+    ATOMIC_SET(p->ip->init_done, 1);
+    /* copy length related parameters back to caller instr */
+    parent_ip->relesing = lcurip->relesing;
+    parent_ip->offbet = lcurip->offbet;
+    parent_ip->offtim = lcurip->offtim;
+    parent_ip->p3 = lcurip->p3;
+    local_ksmps = lcurip->ksmps;
 
-  /* restore globals */
-  csound->ids = saved_ids;
-  csound->curip = parent_ip;
+    /* restore globals */
+    csound->ids = saved_ids;
+    csound->curip = parent_ip;
 
-  /* select perf routine and scale xtratim accordingly */
-  if (local_ksmps != CS_KSMPS) {
-    ksmps_scale = CS_KSMPS / local_ksmps;
-    parent_ip->xtratim = lcurip->xtratim / ksmps_scale;
-    p->h.opadr = (SUBR) useropcd1;
-  }
-  else {
-    parent_ip->xtratim = lcurip->xtratim;
-    p->h.opadr = (SUBR) useropcd2;
-  }
-  if (UNLIKELY(csound->oparms->odebug))
-    csound->Message(csound, "EXTRATIM=> cur(%p): %d, parent(%p): %d\n",
-                    lcurip, lcurip->xtratim, parent_ip, parent_ip->xtratim);
-  return OK;
+    /* select perf routine and scale xtratim accordingly */
+    if (local_ksmps != CS_KSMPS) {
+      ksmps_scale = CS_KSMPS / local_ksmps;
+      parent_ip->xtratim = lcurip->xtratim / ksmps_scale;
+      p->h.opadr = (SUBR) useropcd1;
+    }
+    else {
+      parent_ip->xtratim = lcurip->xtratim;
+      p->h.opadr = (SUBR) useropcd2;
+    }
+    if (UNLIKELY(csound->oparms->odebug))
+      csound->Message(csound, "EXTRATIM=> cur(%p): %d, parent(%p): %d\n",
+                      lcurip, lcurip->xtratim, parent_ip, parent_ip->xtratim);
+    return OK;
 }
 
 /* IV - Sep 17 2002: dummy user opcode function for not initialised case */
@@ -1535,7 +1571,7 @@ int useropcd(CSOUND *csound, UOPCODE *p)
 {
 
   if (UNLIKELY(p->h.nxtp))
-    return csoundPerfError(csound, p->h.insdshead, Str("%s: not initialised"),
+    return csoundPerfError(csound, &(p->h), Str("%s: not initialised"),
                            p->h.optext->t.opcod);
   else
     return OK;
@@ -1590,7 +1626,11 @@ int xoutset(CSOUND *csound, XOUT *p)
     void* in = (void*)p->args[i];
     void* out = (void*)bufs[i];
     tmp[i] = in;
-    current->varType->copyValue(csound, out, in);
+    // DO NOT COPY K or A or F vars
+    if (csoundGetTypeForArg(in) != &CS_VAR_TYPE_K &&
+        csoundGetTypeForArg(in) != &CS_VAR_TYPE_F &&
+        csoundGetTypeForArg(in) != &CS_VAR_TYPE_A)
+      current->varType->copyValue(csound, out, in);
     current = current->next;
   }
 
@@ -1653,18 +1693,37 @@ int setksmpsset(CSOUND *csound, SETKSMPS *p)
 int nstrnumset(CSOUND *csound, NSTRNUM *p)
 {
   /* IV - Oct 31 2002 */
-  *(p->i_insno) = (MYFLT) strarg2insno(csound, p->iname, 0);
-  return (*(p->i_insno) > FL(0.0) ? OK : NOTOK);
+    int res = strarg2insno(csound, p->iname, 0);
+    if (UNLIKELY(res == NOT_AN_INSTRUMENT)) {
+      *p->i_insno = -FL(1.0); return NOTOK;
+    }
+    else {
+      *p->i_insno = (MYFLT)res; return OK;
+    }
 }
 
 int nstrnumset_S(CSOUND *csound, NSTRNUM *p)
 {
   /* IV - Oct 31 2002 */
-  *(p->i_insno) = (MYFLT) strarg2insno(csound,
-                                       ((STRINGDAT *)p->iname)->data, 1);
-  return (*(p->i_insno) > FL(0.0) ? OK : NOTOK);
+    int res = strarg2insno(csound, ((STRINGDAT *)p->iname)->data, 1);
+    if (UNLIKELY(res == NOT_AN_INSTRUMENT)) {
+      *p->i_insno = -FL(1.0); return NOTOK;
+    }
+    else {
+      *p->i_insno = (MYFLT)res; return OK;
+    }
 }
 
+int nstrstr(CSOUND *csound, NSTRSTR *p)
+{
+    char *ss = cs_inverse_hash_get(csound,
+                                   csound->engineState.instrumentNames,
+                                   (int)*p->num);
+    mfree(csound,p->ans->data);
+    p->ans->data = cs_strdup(csound, ss);
+    p->ans->size = strlen(ss);
+    return OK;
+}
 
 /* unlink expired notes from activ chain */
 /*      and mark them inactive           */
@@ -1761,7 +1820,7 @@ int subinstr(CSOUND *csound, SUBINST *p)
   //printf("%s\n", p->ip->strarg);
 
   if (UNLIKELY(p->ip == NULL)) {                /* IV - Oct 26 2002 */
-    return csoundPerfError(csound, p->h.insdshead,
+    return csoundPerfError(csound, &(p->h),
                            Str("subinstr: not initialised"));
   }
   /* copy current spout buffer and clear it */
@@ -2396,8 +2455,8 @@ static void instance(CSOUND *csound, int insno)
       lblbp->prvp = prvpds;
       continue;                               /*    for later refs */
     }
-    // ******** This needs revisipn with no distinction between k- anda- rate ****
-    if ((ep->thread & 07) == 0) {             /* thread 1 OR 2:  */
+    // ******** This needs revisipn with no distinction between k- and a- rate ****
+    if ((ep->thread & 03) == 0) {             /* thread 1 OR 2:  */
       if (ttp->pftype == 'b') {
         prvids = prvids->nxti = opds;
         opds->iopadr = ep->iopadr;
@@ -2414,12 +2473,12 @@ static void instance(CSOUND *csound, int insno)
       if (UNLIKELY(opds->iopadr == NULL))
         csoundDie(csound, Str("null iopadr"));
     }
-    if ((n = ep->thread & 06) != 0) {         /* thread 2 OR 4:   */
+    if ((n = ep->thread & 02) != 0) {         /* thread 2     :   */
       prvpds = prvpds->nxtp = opds;           /* link into pchain */
-      if (!(n & 04) ||
-          ((ttp->pftype == 'k' || ttp->pftype == 'c') && ep->kopadr != NULL))
+      /* if (!(n & 04) || */
+      /*     ((ttp->pftype == 'k' || ttp->pftype == 'c') && ep->kopadr != NULL)) */
         opds->opadr = ep->kopadr;             /*      krate or    */
-      else opds->opadr = ep->aopadr;          /*      arate       */
+      /* else opds->opadr = ep->aopadr;          /\*      arate       *\/ */
       if (UNLIKELY(odebug))
         csound->Message(csound, "opadr = %p\n", (void*) opds->opadr);
       if (UNLIKELY(opds->opadr == NULL))
@@ -2515,28 +2574,27 @@ static void instance(CSOUND *csound, int insno)
 
 int prealloc_(CSOUND *csound, AOP *p, int instname)
 {
-  int     n, a;
+    int     n, a;
 
-  if (instname)
-    n = (int) strarg2opcno(csound, ((STRINGDAT*)p->r)->data, 1,
-                           (*p->b == FL(0.0) ? 0 : 1));
-  else {
-    if (csound->ISSTRCOD(*p->r))
-      n = (int) strarg2opcno(csound, get_arg_string(csound,*p->r), 1,
+    if (instname)
+      n = (int) strarg2opcno(csound, ((STRINGDAT*)p->r)->data, 1,
                              (*p->b == FL(0.0) ? 0 : 1));
-    else n = *p->r;
-  }
+    else {
+      if (csound->ISSTRCOD(*p->r))
+        n = (int) strarg2opcno(csound, get_arg_string(csound,*p->r), 1,
+                               (*p->b == FL(0.0) ? 0 : 1));
+      else n = *p->r;
+    }
 
-  if (UNLIKELY(n < 1))
-    return NOTOK;
-  if(csound->oparms->realtime)
-           csoundSpinLock(&csound->alloc_spinlock);
-  a = (int) *p->a - csound->engineState.instrtxtp[n]->active;
-  for ( ; a > 0; a--)
-    instance(csound, n);
-  if(csound->oparms->realtime)
-          csoundSpinUnLock(&csound->alloc_spinlock);
-  return OK;
+    if (UNLIKELY(n == NOT_AN_INSTRUMENT)) return NOTOK;
+    if (csound->oparms->realtime)
+      csoundSpinLock(&csound->alloc_spinlock);
+    a = (int) *p->a - csound->engineState.instrtxtp[n]->active;
+    for ( ; a > 0; a--)
+      instance(csound, n);
+    if (csound->oparms->realtime)
+      csoundSpinUnLock(&csound->alloc_spinlock);
+    return OK;
 }
 
 int prealloc(CSOUND *csound, AOP *p){
@@ -2559,7 +2617,7 @@ int delete_instr(CSOUND *csound, DELETEIN *p)
   else
     n = (int) (*p->insno + FL(0.5));
 
-  if (UNLIKELY(n < 1 ||
+  if (UNLIKELY(n == NOT_AN_INSTRUMENT ||
                n > csound->engineState.maxinsno ||
                csound->engineState.instrtxtp[n] == NULL))
     return OK;                /* Instrument does not exist so noop */
