@@ -57,6 +57,7 @@
 #include "cs_par_ops.h"
 #include "ugtabs.h"
 #include "compile_ops.h"
+#include "lpred.h"
 
 #define S(x)    sizeof(x)
 
@@ -86,6 +87,9 @@ int32_t modak(CSOUND *, void *);
 int32_t addaa(CSOUND *, void *), subaa(CSOUND *, void *);
 int32_t mulaa(CSOUND *, void *), divaa(CSOUND *, void *);
 int32_t modaa(CSOUND *, void *);
+int32_t addin(CSOUND *, void *), addina(CSOUND *, void *);
+int32_t subin(CSOUND *, void *), subina(CSOUND *, void *);
+int32_t addinak(CSOUND *, void *), subinak(CSOUND *, void *);
 int32_t divzkk(CSOUND *, void *), divzka(CSOUND *, void *);
 int32_t divzak(CSOUND *, void *), divzaa(CSOUND *, void *);
 int32_t int1(CSOUND *, void *), int1a(CSOUND *, void *);
@@ -94,6 +98,7 @@ int32_t int1_round(CSOUND *, void *), int1a_round(CSOUND *, void *);
 int32_t int1_floor(CSOUND *, void *), int1a_floor(CSOUND *, void *);
 int32_t int1_ceil(CSOUND *, void *), int1a_ceil(CSOUND *, void *);
 int32_t rnd1(CSOUND *, void *), birnd1(CSOUND *, void *);
+int32_t rnd1seed(CSOUND *, void *);
 int32_t abs1(CSOUND *, void *), exp01(CSOUND *, void *);
 int32_t log01(CSOUND *, void *), sqrt1(CSOUND *, void *);
 int32_t sin1(CSOUND *, void *), cos1(CSOUND *, void *);
@@ -192,6 +197,8 @@ int32_t rndset(CSOUND *, void *), krand(CSOUND *, void *);
 int32_t arand(CSOUND *, void *), rhset(CSOUND *, void *);
 int32_t krandh(CSOUND *, void *), randh(CSOUND *, void *);
 int32_t riset(CSOUND *, void *), krandi(CSOUND *, void *);
+int32_t rcset(CSOUND *, void *), randc(CSOUND *, void *);
+int32_t krandc(CSOUND *, void *);
 int32_t randi(CSOUND *, void *), rndset2(CSOUND *, void *);
 int32_t krand2(CSOUND *, void *), arand2(CSOUND *, void *);
 int32_t rhset2(CSOUND *, void *), krandh2(CSOUND *, void *);
@@ -388,9 +395,8 @@ int32_t useropcdset(CSOUND *, void *), useropcd(CSOUND *, void *);
 int32_t setksmpsset(CSOUND *, void *);
 int32_t xinset(CSOUND *, void *), xoutset(CSOUND *, void *);
 int32_t ingoto(CSOUND *, void *), kngoto(CSOUND *, void *);
-int32_t iingoto(CSOUND *, void *), kingoto(CSOUND *, void *);
 int32_t nstrnumset(CSOUND *, void *), turnoff2k(CSOUND *, void *);
-int32_t nstrnumset_S(CSOUND *, void *);
+int32_t nstrnumset_S(CSOUND *, void *), nstrstr(CSOUND *, void *);
 int32_t turnoff2S(CSOUND *, void *) ;
 int32_t loop_l_i(CSOUND *, void *), loop_le_i(CSOUND *, void *);
 int32_t loop_g_i(CSOUND *, void *), loop_ge_i(CSOUND *, void *);
@@ -463,3 +469,26 @@ int32_t losset_phs(CSOUND *, void *);
 int32_t loscil_phs(CSOUND *, void *);
 int32_t loscil3_phs(CSOUND *, void *);
 int32_t balance2(CSOUND *, void *);
+int32_t gauss_scalar(CSOUND *csound, void *p);
+int32_t gauss_vector(CSOUND *csound, void *p);
+int32_t lpfil_init(CSOUND *csound, void *p);
+int32_t lpfil_perf(CSOUND *csound, void *p);
+int32_t lpfil2_init(CSOUND *csound, void *p);
+int32_t lpfil2_perf(CSOUND *csound, void *p);
+int32_t lpred_run(CSOUND *csound, void *p);
+int32_t lpred_alloc(CSOUND *csound, void *p);
+int32_t lpred_i(CSOUND *csound, void *p);
+int32_t lpfil3_init(CSOUND *csound, void *p);
+int32_t lpfil3_perf(CSOUND *csound, void *p);
+int32_t lpred_run2(CSOUND *csound, void *p);
+int32_t lpred_alloc2(CSOUND *csound, void *p);
+int32_t lpcpvs(CSOUND *csound, void *p);
+int32_t lpcpvs_init(CSOUND *csound, void *p);
+int32_t pvscoefs_init(CSOUND *csound, void *p);
+int32_t pvscoefs(CSOUND *csound, void *p);
+int32_t coef2parm_init(CSOUND *csound, void *p);
+int32_t coef2parm(CSOUND *csound, void *p);
+int32_t resonbnk_init(CSOUND *csound, void *p);
+int32_t resonbnk(CSOUND *csound, void *p);
+int32_t schedule_array(CSOUND *csound, void *p);
+
