@@ -17,8 +17,8 @@
 
     You should have received a copy of the GNU Lesser General Public
     License along with Csound; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-    02111-1307 USA
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+    02110-1301 USA
 */
 
 #include "csoundCore.h"                         /*      WINDOW.C        */
@@ -106,19 +106,21 @@ void dispset(CSOUND *csound,            /* setup a new window       */
     char *t = wdptr->caption;
     char *tlim = t + CAPSIZE - 1;
 
-    csound->GetOParms(csound, &O);
-    if (!O.displays) return;    /* return if displays disabled */
-    wdptr->fdata    = fdata;            /* init remainder of data structure   */
+
+    csound->GetOParms(csound, &O);    
+    if (!O.displays) return;    // return if displays disabled 
+    wdptr->fdata    = fdata;            // init remainder of data structure  
     wdptr->npts     = npts;
     while (*s != '\0' && t < tlim)
-      *t++ = *s++;                      /*  (copy the caption) */
+      *t++ = *s++;                      //  (copy the caption) 
     *t = '\0';
-    if (!wdptr->windid) {   /* if no window defined for this str, create one */
+    // if no window defined for this str, create one
+    if (!wdptr->windid && csound->csoundMakeGraphCallback_ != NULL) {   
       csound->csoundMakeGraphCallback_(csound, wdptr, label);
       if (O.postscript)
-        PS_MakeGraph(csound, wdptr, label); /* open PS file + write header    */
-    }
-
+        PS_MakeGraph(csound, wdptr, label); 
+    } 
+  
     wdptr->waitflg  = waitflg;
     wdptr->polarity = (int16)NOPOL;
     wdptr->max      = FL(0.0);
@@ -126,6 +128,7 @@ void dispset(CSOUND *csound,            /* setup a new window       */
     wdptr->absmax   = FL(0.0);
     wdptr->oabsmax  = FL(0.0);
     wdptr->danflag  = 0;
+    
 }
 
 int dispexit(CSOUND *csound)
@@ -153,6 +156,7 @@ void display(CSOUND *csound, WINDAT *wdptr)   /* prepare a MYFLT array, then  */
 
     if (!O.displays)  return;   /* displays disabled? return */
     fp = wdptr->fdata;
+    if(fp == NULL) return;
     fplim = fp + wdptr->npts;
     for (max = *fp++, min = max; fp < fplim; ) {  /* find max & min values */
       if ((fval = *fp++) > max)       max = fval;
