@@ -150,7 +150,7 @@ int32_t kdsplay(CSOUND *csound, DSPLAY *p)
     p->nxtp = fp;
     return OK;
  err1:
-    return csound->PerfError(csound, p->h.insdshead,
+    return csound->PerfError(csound, &(p->h),
                              Str("display: not initialised"));
 }
 
@@ -246,10 +246,11 @@ int32_t fftset(CSOUND *csound, DSPFFT *p) /* fftset, dspfft -- calc Fast Fourier
 
     window_size = (int32_t)*p->inpts;
     if (UNLIKELY(window_size > WINDMAX)) {
-      return csound->InitError(csound, Str("too many points requested"));
+      return csound->InitError(csound, Str("too many points requested (%d)"), window_size);
     }
     if (UNLIKELY(window_size < WINDMIN)) {
-      return csound->InitError(csound, Str("too few points requested"));
+      return csound->InitError(csound, Str("too few points requested (%d), minimum is %d"),
+                               window_size, WINDMIN);
     }
     if (UNLIKELY(window_size < 1L || (window_size & (window_size - 1L)) != 0L)) {
       return csound->InitError(csound, Str("window size must be power of two"));
@@ -402,7 +403,7 @@ int32_t kdspfft(CSOUND *csound, DSPFFT *p)
     p->bufp = bufp;
     return OK;
  err1:
-    return csound->PerfError(csound, p->h.insdshead,
+    return csound->PerfError(csound, &(p->h),
                              Str("dispfft: not initialised"));
 }
 
@@ -454,7 +455,7 @@ int32_t dspfft(CSOUND *csound, DSPFFT *p)
     p->bufp = bufp;
     return OK;
  err1:
-    return csound->PerfError(csound, p->h.insdshead,
+    return csound->PerfError(csound, &(p->h),
                              Str("dispfft: not initialised"));
 }
 
@@ -737,6 +738,6 @@ int32_t tempest(CSOUND *csound, TEMPEST *p)
     *p->kout = p->tempo;                    /* put current tempo */
     return OK;
  err1:
-      return csound->PerfError(csound, p->h.insdshead,
+    return csound->PerfError(csound, &(p->h),
                                Str("tempest: not initialised"));
 }
