@@ -133,7 +133,7 @@ static int32_t getcurdir(CSOUND *csound, GETCWD *p)
 }
 
 #ifndef MAXLINE
-#define MAXLINE 1024
+#define MAXLINE 8192
 #endif
 
 typedef struct {
@@ -186,8 +186,8 @@ static int32_t readf_init_S(CSOUND *csound, READF *p){
 static int32_t readf(CSOUND *csound, READF *p)
 {
     p->Sline->data[0] = '\0';
-    if (UNLIKELY(fgets(p->Sline->data,
-                       p->Sline->size-1, p->fd)==NULL)) {
+    if (UNLIKELY(p->fd && (fgets(p->Sline->data,
+                                 p->Sline->size-1, p->fd)==NULL))) {
       int32_t ff = feof(p->fd);
       fclose(p->fd);
       p->fd = NULL;
@@ -196,7 +196,7 @@ static int32_t readf(CSOUND *csound, READF *p)
         return OK;
       }
       else
-        return csound->PerfError(csound, p->h.insdshead,
+        return csound->PerfError(csound, &(p->h),
                                  "%s", Str("readf: read failure"));
     }
     *p->line = ++p->lineno;
