@@ -858,10 +858,13 @@ static int32_t infile_set_(CSOUND *csound, INFILE *p, int32_t istring)
     p->flag = 1;
     memset(&sfinfo, 0, sizeof(SF_INFO));
     sfinfo.samplerate = (int32_t) MYFLT2LRND(CS_ESR);
-    if ((int32_t) MYFLT2LRND(*p->iflag) == 0)
+    /* Following code is seriously broken*/
+    if ((int32_t) MYFLT2LRND(*p->iflag) == -2)
       sfinfo.format = FORMAT2SF(AE_FLOAT) | TYPE2SF(TYP_RAW);
-    else
+    else if ((int32_t) MYFLT2LRND(*p->iflag) == -1)
       sfinfo.format = FORMAT2SF(AE_SHORT) | TYPE2SF(TYP_RAW);
+    else
+      sfinfo.format = 0;
     sfinfo.channels = p->INOCOUNT - 3;
      if (CS_KSMPS >= 512)
       p->frames = CS_KSMPS;
@@ -905,19 +908,7 @@ static int32_t infile_set_S(CSOUND *csound, INFILE *p){
     return infile_set_(csound,p,1);
 }
 
-static inline void tabensure(CSOUND *csound, ARRAYDAT *p, int32_t size)
-{
-    if (p->data==NULL || p->dimensions == 0 ||
-        (p->dimensions==1 && p->sizes[0] < size)) {
-      uint32_t ss = sizeof(MYFLT)*size;
-      if (p->data==NULL) p->data = (MYFLT*)csound->Malloc(csound, ss);
-      else p->data = (MYFLT*) csound->ReAlloc(csound, p->data, ss);
-      p->dimensions = 1;
-      p->arrayMemberSize = sizeof(MYFLT);
-      p->sizes = (int32_t*)csound->Malloc(csound, sizeof(int32_t));
-      p->sizes[0] = size;
-    }
-}
+#include "arrays.h"
 
 static int32_t infile_set_A(CSOUND *csound, INFILEA *p)
 {
@@ -927,10 +918,12 @@ static int32_t infile_set_A(CSOUND *csound, INFILEA *p)
     p->flag = 1;
     memset(&sfinfo, 0, sizeof(SF_INFO));
     sfinfo.samplerate = (int32_t) MYFLT2LRND(CS_ESR);
-    if ((int32_t) MYFLT2LRND(*p->iflag) == 0)
+    if ((int32_t) MYFLT2LRND(*p->iflag) == -2)
       sfinfo.format = FORMAT2SF(AE_FLOAT) | TYPE2SF(TYP_RAW);
-    else
+    else if ((int32_t) MYFLT2LRND(*p->iflag) == -1)
       sfinfo.format = FORMAT2SF(AE_SHORT) | TYPE2SF(TYP_RAW);
+    else
+      sfinfo.format = 0;
     sfinfo.channels = p->INOCOUNT - 3;
     if (CS_KSMPS >= 512)
       p->frames = CS_KSMPS;
@@ -961,7 +954,7 @@ static int32_t infile_set_A(CSOUND *csound, INFILEA *p)
     if (p->f.async == 1)
     csound->FSeekAsync(csound,p->f.fd, p->currpos*p->f.nchnls, SEEK_SET);
 
-    tabensure(csound, p->tabout, p->chn);
+    tabinit(csound, p->tabout, p->chn);
     return OK;
 }
 
@@ -1083,10 +1076,12 @@ static int32_t kinfile_set_(CSOUND *csound, KINFILE *p, int32_t istring)
 
     memset(&sfinfo, 0, sizeof(SF_INFO));
     sfinfo.samplerate = (int32_t) MYFLT2LRND(CS_EKR);
-    if ((int32_t) MYFLT2LRND(*p->iflag) == 0)
+    if ((int32_t) MYFLT2LRND(*p->iflag) == -2)
       sfinfo.format = FORMAT2SF(AE_FLOAT) | TYPE2SF(TYP_RAW);
-    else
+    else if ((int32_t) MYFLT2LRND(*p->iflag) == -1)
       sfinfo.format = FORMAT2SF(AE_SHORT) | TYPE2SF(TYP_RAW);
+    else
+      sfinfo.format = 0;
     sfinfo.channels = p->INOCOUNT - 3;
 
     p->nargs = p->INOCOUNT - 3;
@@ -1303,7 +1298,7 @@ static int32_t fprintf_set_(CSOUND *csound, FPRINTF *p, int32_t istring)
                          p->fname, istring, "w", 1);
     if (UNLIKELY(n < 0))
       return NOTOK;
-    setvbuf(p->f.f, (char*)NULL, _IOLBF, 0); /* Seems a good option */
+    //setvbuf(p->f.f, (char*)NULL, _IOLBF, BUFSIZ); /* Seems a good option */
     /* Copy the string to the storage place in PRINTKS.
      *
      * We will look out for certain special codes and write special
@@ -1447,7 +1442,9 @@ void sprints1(char *outstring,  char *fmt, MYFLT **kvals, int32 numVals)
           case 'l':
             snprintf(outstring, len, strseg, (int32) MYFLT2LRND(*kvals[j]));
             break;
-
+          case 's':
+            snprintf(outstring, len, strseg, ((STRINGDAT*)(kvals[j]))->data);
+            break;
           default:
             snprintf(outstring, len, strseg, *kvals[j]);
             break;
@@ -1516,7 +1513,7 @@ static int32_t fprintf_k(CSOUND *csound, FPRINTF *p)
     (void) csound;
     sprints1(string, p->txtstring, p->argums, p->INOCOUNT - 2);
     fprintf(p->f.f, "%s", string);
-
+    fflush(p->f.f);
     return OK;
 }
 
@@ -1529,7 +1526,7 @@ static int32_t fprintf_i(CSOUND *csound, FPRINTF *p)
       return NOTOK;
     sprints1(string, p->txtstring, p->argums, p->INOCOUNT - 2);
     fprintf(p->f.f,"%s", string);
-    /* fflush(p->f.f); */
+    fflush(p->f.f);
     return OK;
 }
 
@@ -1541,25 +1538,24 @@ static int32_t fprintf_i_S(CSOUND *csound, FPRINTF *p)
       return NOTOK;
     sprints1(string, p->txtstring, p->argums, p->INOCOUNT - 2);
     fprintf(p->f.f, "%s", string);
-    /* fflush(p->f.f); */
+    fflush(p->f.f);
     return OK;
 }
 
 #define S(x)    sizeof(x)
 static OENTRY localops[] = {
 
-    {"fprints",    S(FPRINTF),      0, 1,  "",     "SSM",
+    {"fprints",    S(FPRINTF),      0, 1,  "",     "SSN",
         (SUBR) fprintf_i_S, (SUBR) NULL,(SUBR) NULL, NULL, },
-        {"fprints.i",    S(FPRINTF),      0, 1,  "",     "iSM",
+    {"fprints.i",    S(FPRINTF),      0, 1,  "",     "iSN",
         (SUBR) fprintf_i, (SUBR) NULL,(SUBR) NULL, NULL},
-    { "fprintks",   S(FPRINTF),    WR, 3,  "",     "SSM",
+    { "fprintks",   S(FPRINTF),    WR, 3,  "",     "SSN",
         (SUBR) fprintf_set_S,     (SUBR) fprintf_k,   (SUBR) NULL, NULL,},
-    { "fprintks.i",   S(FPRINTF),    WR, 3,  "",     "iSM",
+    { "fprintks.i",   S(FPRINTF),    WR, 3,  "",     "iSN",
         (SUBR) fprintf_set,     (SUBR) fprintf_k,   (SUBR) NULL, NULL},
-     { "vincr",      S(INCR),        0, 2,  "",     "aa",
+     { "vincr",      S(INCR),       WI, 2,  "",     "aa",
         (SUBR) NULL,            (SUBR) incr, NULL         },
-    { "clear",      S(CLEARS),      0, 2,  "",     "y",
-
+    { "clear",      S(CLEARS),      WI, 2,  "",     "y",
         (SUBR) NULL,            (SUBR) clear, NULL},
     { "fout",       S(OUTFILE),     0, 3,  "",     "Siy",
         (SUBR) outfile_set_S,     (SUBR) outfile, NULL},
@@ -1569,35 +1565,33 @@ static OENTRY localops[] = {
     /*     (SUBR) outfile_set,     (SUBR) outfile, NULL}, */
     { "foutk",      S(KOUTFILE),    0, 3,  "",     "Siz",
         (SUBR) koutfile_set_S,    (SUBR) koutfile,    (SUBR) NULL, NULL },
-        { "foutk.i",      S(KOUTFILE),    0, 3,  "",     "iiz",
+    { "foutk.i",      S(KOUTFILE),    0, 3,  "",     "iiz",
         (SUBR) koutfile_set,    (SUBR) koutfile,    (SUBR) NULL, NULL },
-
     { "fouti",      S(IOUTFILE),    0, 1,  "",     "iiim",
         (SUBR) ioutfile_set,    (SUBR) NULL,        (SUBR) NULL, NULL         },
     { "foutir",     S(IOUTFILE_R),  0, 3,  "",     "iiim",
-
         (SUBR) ioutfile_set_r,  (SUBR) ioutfile_r,  (SUBR) NULL, NULL},
     { "fiopen",     S(FIOPEN),      0, 1,  "i",    "Si",
         (SUBR) fiopen_S,          (SUBR) NULL,        (SUBR) NULL, NULL},
-        { "fiopen.i",     S(FIOPEN),      0, 1,  "i",    "ii",
+    { "fiopen.i",     S(FIOPEN),      0, 1,  "i",    "ii",
         (SUBR) fiopen,          (SUBR) NULL,        (SUBR) NULL, NULL},
     { "ficlose",    S(FICLOSE),     0, 1,  "",     "S",
         (SUBR) ficlose_opcode_S,  (SUBR) NULL,        (SUBR) NULL, NULL},
     { "ficlose.S",  S(FICLOSE),     0, 1,  "",     "i",
         (SUBR) ficlose_opcode,  (SUBR) NULL,        (SUBR) NULL, NULL },
-    { "fin.a",      S(INFILE),      0, 3,  "",      "Siiy",
+    { "fin.a",      S(INFILE),     WI|_QQ, 3,  "",      "Siiy",
         (SUBR) infile_set_S,    (SUBR) infile_act, NULL},
-    { "fin.A",      S(INFILEA),      0, 3,  "",     "Siia[]",
+    { "fin.A",      S(INFILEA),    WI, 3,  "",     "Siia[]",
         (SUBR) infile_set_A,    (SUBR) infile_arr, NULL},
-    { "fin.i",      S(INFILE),       0, 3,  "",     "iiiy",
+    { "fin.i",      S(INFILE),     WI|_QQ, 3,  "",     "iiiy",
         (SUBR) infile_set,      (SUBR) infile_act, NULL},
-    { "fink",       S(KINFILE),      0, 3,  "",     "Siiz",
+    { "fink",       S(KINFILE),    WI, 3,  "",     "Siiz",
         (SUBR) kinfile_set_S,     (SUBR) kinfile,     (SUBR) NULL, NULL},
-    { "fink.i",       S(KINFILE),    0, 3,  "",     "iiiz",
+    { "fink.i",       S(KINFILE),  WI, 3,  "",     "iiiz",
         (SUBR) kinfile_set,     (SUBR) kinfile,     (SUBR) NULL, NULL},
-    { "fini",       S(I_INFILE),    0, 1,  "",     "Siim",
+    { "fini",       S(I_INFILE),   WI, 1,  "",     "Siim",
       (SUBR) i_infile_S,        (SUBR) NULL,        (SUBR) NULL, NULL },
-    { "fini.i",       S(I_INFILE),    0, 1,  "",     "iiim",
+    { "fini.i",       S(I_INFILE), WI, 1,  "",     "iiim",
         (SUBR) i_infile,        (SUBR) NULL,        (SUBR) NULL, NULL}
 };
 
