@@ -1,18 +1,4 @@
 /*
-    ftgen.c:
-
-    Copyright (C) 1991, 1994, 1995, 1998, 2000, 2004
-                  Barry Vercoe, John ffitch, Paris Smaragdis,
-                  Gabriel Maldonado, Richard Karpen, Greg Sullivan,
-                  Pete Moss, Istvan Varga, Victor Lazzarini
-
-    This file is part of Csound.
-
-    The Csound Library is free software; you can redistribute it
-    and/or modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
-
     Csound is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -32,17 +18,17 @@
 
 typedef struct {
     OPDS    h;
-    MYFLT   *ifno, *p1, *p2, *p3, *p4, *p5, *argums[VARGMAX];
+    MYFLT   *ifno, *p1, *p2, *p3, *p4, *p5, *argums[VARGMAX-5];
 } FTGEN;
 
 typedef struct {
     OPDS    h;
-    MYFLT   *ifilno, *iflag, *argums[VARGMAX];
+    MYFLT   *ifilno, *iflag, *argums[VARGMAX-2];
 } FTLOAD;  /* gab 30 jul 2002 */
 
 typedef struct {
     OPDS    h;
-    MYFLT   *ifilno, *ktrig, *iflag, *argums[VARGMAX];
+    MYFLT   *ifilno, *ktrig, *iflag, *argums[VARGMAX-2];
     FTLOAD  p;
 } FTLOAD_K; /* gab 30 jul 2002 */
 
@@ -102,43 +88,43 @@ static int32_t ftgen_(CSOUND *csound, FTGEN *p, int32_t istring1, int32_t istrin
     fp[4] = *p->p4;
 
 
-      if (istring1) {              /* Named gen */
-        NAMEDGEN *named = (NAMEDGEN*) csound->GetNamedGens(csound);
-        while (named) {
-          if (strcmp(named->name, ((STRINGDAT *) p->p4)->data) == 0) {
-            /* Look up by name */
-            fp[4] = named->genum;
-            break;
-          }
-          named = named->next;                            /*  and round again   */
-        }
-        if (UNLIKELY(named == NULL)) {
-          csound->Free(csound,ftevt);
-          return csound->InitError(csound,
-                                   Str("Named gen \"%s\" not defined"),
-                                   (char *)p->p4);
-        }
-        else fp[4] = named->genum;
-      }
-
-      if (istring2) {  /* string argument: */
-        n = (int32_t) fp[4];
-        fp[5] = SSTRCOD;
-        if (n < 0)
-          n = -n;
-        switch (n) {                      /*   must be Gen01, 23, 28, 43, 49 */
-        case 1:
-        case 23:
-        case 28:
-        case 43:
-        case 49:
-          ftevt->strarg = ((STRINGDAT *) p->p5)->data;
+    if (istring1) {              /* Named gen */
+      NAMEDGEN *named = (NAMEDGEN*) csound->GetNamedGens(csound);
+      while (named) {
+        if (strcmp(named->name, ((STRINGDAT *) p->p4)->data) == 0) {
+          /* Look up by name */
+          fp[4] = named->genum;
           break;
-        default:
-          csound->Free(csound, ftevt);
-          return csound->InitError(csound, Str("ftgen string arg not allowed"));
         }
+        named = named->next;                            /*  and round again   */
       }
+      if (UNLIKELY(named == NULL)) {
+        csound->Free(csound,ftevt);
+        return csound->InitError(csound,
+                                 Str("Named gen \"%s\" not defined"),
+                                 (char *)p->p4);
+      }
+      // else fp[4] = named->genum;
+    }
+
+    if (istring2) {  /* string argument: */
+      n = (int32_t) fp[4];
+      fp[5] = SSTRCOD;
+      if (n < 0)
+        n = -n;
+      switch (n) {                      /*   must be Gen01, 23, 28, 43, 49 */
+      case 1:
+      case 23:
+      case 28:
+      case 43:
+      case 49:
+        ftevt->strarg = ((STRINGDAT *) p->p5)->data;
+        break;
+      default:
+        csound->Free(csound, ftevt);
+        return csound->InitError(csound, Str("ftgen string arg not allowed"));
+      }
+    }
     else {
       fp[5] = *p->p5;                                   /* else no string */
     }
@@ -244,7 +230,7 @@ static int32_t ftfree(CSOUND *csound, FTFREE *p)
     return register_ftable_delete(csound, p, fno);
 }
 
-static int32_t myInitError(CSOUND *csound, INSDS *p, const char *str, ...)
+static int32_t myInitError(CSOUND *csound, OPDS *p, const char *str, ...)
 {
     IGN(p);
     return csound->InitError(csound, "%s",str);
@@ -257,17 +243,17 @@ static int32_t ftload_(CSOUND *csound, FTLOAD *p, int32_t istring)
     char  filename[MAXNAME];
     int32_t   nargs = csound->GetInputArgCnt(p) - 2;
     FILE  *file = NULL;
-    int32_t   (*err_func)(CSOUND *, INSDS *, const char *, ...);
+    int32_t   (*err_func)(CSOUND *, OPDS *, const char *, ...);
     FUNC  *(*ft_func)(CSOUND *, MYFLT *);
     void  *fd;
 
-    if (strncmp(csound->GetOpcodeName(p), "ftload", 6) != 0) {
+    if (strncmp(csound->GetOpcodeName(p), "ftloadk", 7) == 0) {
       nargs--;
       ft_func = csound->FTFindP;
       err_func = csound->PerfError;
     }
     else {
-      ft_func = csound->FTnp2Find;
+      ft_func = csound->FTnp2Finde;
       err_func = myInitError;
     }
 
@@ -451,15 +437,15 @@ static int32_t ftload_(CSOUND *csound, FTLOAD *p, int32_t istring)
     return OK;
  err:
     csound->FileClose(csound, fd);
-    return err_func(csound, p->h.insdshead,
+    return err_func(csound, &(p->h),
                     Str("ftload: error allocating ftable"));
  err2:
-    return err_func(csound, p->h.insdshead, Str("ftload: no table numbers"));
+    return err_func(csound, &(p->h), Str("ftload: no table numbers"));
  err3:
-    return err_func(csound, p->h.insdshead, Str("ftload: unable to open file"));
+    return err_func(csound, &(p->h), Str("ftload: unable to open file"));
  err4:
     csound->FileClose(csound, fd);
-    return err_func(csound, p->h.insdshead, Str("ftload: incorrect file"));
+    return err_func(csound, &(p->h), Str("ftload: incorrect file"));
 }
 
 static int32_t ftload(CSOUND *csound, FTLOAD *p)
@@ -493,7 +479,7 @@ static int32_t ftsave_(CSOUND *csound, FTLOAD *p, int32_t istring)
     char  filename[MAXNAME];
     int32_t   nargs = csound->GetInputArgCnt(p) - 3;
     FILE  *file = NULL;
-    int32_t   (*err_func)(CSOUND *, INSDS *, const char *, ...);
+    int32_t   (*err_func)(CSOUND *, OPDS *, const char *, ...);
     FUNC  *(*ft_func)(CSOUND *, MYFLT *);
     void  *fd;
 
@@ -503,7 +489,7 @@ static int32_t ftsave_(CSOUND *csound, FTLOAD *p, int32_t istring)
     }
     else {
       nargs = csound->GetInputArgCnt(p) - 2;
-      ft_func = csound->FTnp2Find;
+      ft_func = csound->FTnp2Finde;
       err_func = myInitError;
     }
 
@@ -596,15 +582,15 @@ static int32_t ftsave_(CSOUND *csound, FTLOAD *p, int32_t istring)
     return OK;
  err:
     csound->FileClose(csound, fd);
-    return err_func(csound, p->h.insdshead,
+    return err_func(csound, &(p->h),
                     Str("ftsave: Bad table number. Saving is possible "
                         "only for existing tables."));
  err2:
-    return err_func(csound, p->h.insdshead, Str("ftsave: no table numbers"));
+    return err_func(csound, &(p->h), Str("ftsave: no table numbers"));
  err3:
-    return err_func(csound, p->h.insdshead, Str("ftsave: unable to open file"));
+    return err_func(csound, &(p->h), Str("ftsave: unable to open file"));
  err4:
-    return err_func(csound, p->h.insdshead, Str("ftsave: failed to write file"));
+    return err_func(csound, &(p->h), Str("ftsave: failed to write file"));
 }
 
 static int32_t ftsave(CSOUND *csound, FTLOAD *p){
@@ -659,39 +645,34 @@ static int32_t ftgen_list(CSOUND *csound, FTGEN *p, int32_t istring)
     fp[4] = *p->p4;
 
 
-      if (istring) {              /* Named gen */
-        NAMEDGEN *named = (NAMEDGEN*) csound->GetNamedGens(csound);
-        while (named) {
-          if (strcmp(named->name, ((STRINGDAT *) p->p4)->data) == 0) {
-            /* Look up by name */
-            fp[4] = named->genum;
-           break;
-          }
-          named = named->next;                            /*  and round again   */
+    if (istring) {              /* Named gen */
+      NAMEDGEN *named = (NAMEDGEN*) csound->GetNamedGens(csound);
+      while (named) {
+        if (strcmp(named->name, ((STRINGDAT *) p->p4)->data) == 0) {
+          /* Look up by name */
+          fp[4] = named->genum;
+          break;
         }
-        if (UNLIKELY(named == NULL)) {
-          csound->Free(csound,ftevt);
-          return csound->InitError(csound,
-                                   Str("Named gen \"%s\" not defined"),
-                                   (char *)p->p4);
-        }
-        else fp[4] = named->genum;
+        named = named->next;                            /*  and round again   */
       }
+      if (UNLIKELY(named == NULL)) {
+        csound->Free(csound,ftevt);
+        return csound->InitError(csound,
+                                 Str("Named gen \"%s\" not defined"),
+                                 (char *)p->p4);
+      }
+    }
 
-      ARRAYDAT *array = (ARRAYDAT*) (p->p5);
-      n = array->sizes[0];
-      ftevt->pcnt = (int16) n+4;
-      int32_t i = 0;
-      memcpy(&fp[5], array->data, n*sizeof(MYFLT));
-      while (i < n) {
-        i++;
-      }
-      n = csound->hfgens(csound, &ftp, ftevt, 1);         /* call the fgen */
-     csound->Free(csound,ftevt);
-     if (UNLIKELY(n != 0))
-       return csound->InitError(csound, Str("ftgen error"));
-     if (ftp != NULL)
-       *p->ifno = (MYFLT) ftp->fno;                      /* record the fno */
+    ARRAYDAT *array = (ARRAYDAT*) (p->p5);
+    n = array->sizes[0];
+    ftevt->pcnt = (int16) n+4;
+    memcpy(&fp[5], array->data, n*sizeof(MYFLT));
+    n = csound->hfgens(csound, &ftp, ftevt, 1);         /* call the fgen */
+    csound->Free(csound,ftevt);
+    if (UNLIKELY(n != 0))
+      return csound->InitError(csound, Str("ftgen error"));
+    if (ftp != NULL)
+      *p->ifno = (MYFLT) ftp->fno;                      /* record the fno */
     return OK;
 }
 
@@ -719,15 +700,15 @@ static OENTRY localops[] = {
   { "ftfree",   S(FTFREE),    TW, 1,  "",   "ii",     (SUBR) ftfree, NULL, NULL   },
   { "ftsave",   S(FTLOAD),    TR, 1,  "",   "iim",    (SUBR) ftsave, NULL, NULL   },
   { "ftsave.S",   S(FTLOAD),  TR, 1,  "",   "Sim",    (SUBR) ftsave_S, NULL, NULL },
-  { "ftload",   S(FTLOAD),    TR, 1,  "",   "iim",    (SUBR) ftload, NULL, NULL   },
-    { "ftload.S",  S(FTLOAD), TR, 1,  "",   "Sim",    (SUBR) ftload_S, NULL, NULL },
+  { "ftload",   S(FTLOAD),    TW, 1,  "",   "iim",    (SUBR) ftload, NULL, NULL   },
+  { "ftload.S",  S(FTLOAD), TW, 1,  "",   "Sim",    (SUBR) ftload_S, NULL, NULL },
   { "ftsavek",  S(FTLOAD_K),  TW, 3,  "",   "ikim",   (SUBR) ftsave_k_set,
                                                   (SUBR) ftsave_k, NULL       },
   { "ftsavek.S",  S(FTLOAD_K),  TW, 3,  "",   "Skim",   (SUBR) ftsave_k_set,
                                                   (SUBR) ftsave_kS, NULL       },
   { "ftloadk",  S(FTLOAD_K),  TW, 3,  "",   "ikim",   (SUBR) ftsave_k_set,
     (SUBR) ftload_k, NULL       },
- { "ftloadk.S",  S(FTLOAD_K),  TW, 3,  "",   "Skim",   (SUBR) ftsave_k_set,
+  { "ftloadk.S",  S(FTLOAD_K),  TW, 3,  "",   "Skim",   (SUBR) ftsave_k_set,
                                                   (SUBR) ftload_kS, NULL       }
 };
 
