@@ -46,6 +46,9 @@ static int32_t mtable_i(CSOUND *csound,MTABLEI *p)
       int64_t indx = (int64_t) fndx;
       MYFLT fract = fndx - indx;
       for (j=0; j < nargs; j++) {
+        if (UNLIKELY((indx + 1) * nargs + j >= ftp->flen + 1)) {
+          return csound->InitError(csound, Str("vtablei: reading past end of table"));
+        }
         v1 = table[indx * nargs + j];
         v2 = table[(indx + 1) * nargs + j];
         **out++ = v1 + (v2 - v1) * fract;
@@ -84,7 +87,7 @@ static int32_t mtable_k(CSOUND *csound,MTABLE *p)
     if (p->pfn != (int64_t)*p->xfn) {
       FUNC *ftp;
       if (UNLIKELY( (ftp = csound->FTnp2Find(csound, p->xfn) ) == NULL)) {
-        return csound->PerfError(csound, p->h.insdshead,
+        return csound->PerfError(csound, &(p->h),
                                  Str("vtablek: incorrect table number"));
       }
       p->pfn = (int64_t)*p->xfn;
@@ -138,7 +141,7 @@ static int32_t mtable_a(CSOUND *csound,MTABLE *p)
     if (p->pfn != (int64_t)*p->xfn) {
       FUNC *ftp;
       if (UNLIKELY( (ftp = csound->FTnp2Find(csound, p->xfn) ) == NULL)) {
-        return csound->PerfError(csound, p->h.insdshead,
+        return csound->PerfError(csound, &(p->h),
                                  Str("vtablea: incorrect table number"));
       }
       p->pfn = (int64_t)*p->xfn;
@@ -314,7 +317,7 @@ static int32_t mtablew_k(CSOUND *csound,MTABLEW *p)
     if (p->pfn != (int64_t)*p->xfn) {
       FUNC *ftp;
       if (UNLIKELY( (ftp = csound->FTnp2Find(csound, p->xfn) ) == NULL)) {
-        return csound->PerfError(csound, p->h.insdshead,
+        return csound->PerfError(csound, &(p->h),
                                  Str("vtablewk: incorrect table number"));
       }
       p->pfn = (int64_t)*p->xfn;
@@ -347,7 +350,7 @@ static int32_t mtablew_a(CSOUND *csound,MTABLEW *p)
     if (p->pfn != (int64_t)*p->xfn) {
       FUNC *ftp;
       if (UNLIKELY( (ftp = csound->FTnp2Find(csound, p->xfn) ) == NULL)) {
-        return csound->PerfError(csound, p->h.insdshead,
+        return csound->PerfError(csound, &(p->h),
                                  Str("vtablewa: incorrect table number"));
       }
       p->pfn = (int64_t)*p->xfn;
@@ -411,7 +414,7 @@ static int32_t mtabw_k(CSOUND *csound,MTABW *p)
     if (p->pfn != (int64_t)*p->xfn) {
       FUNC *ftp;
       if (UNLIKELY( (ftp = csound->FTnp2Find(csound, p->xfn) ) == NULL)) {
-        return csound->PerfError(csound, p->h.insdshead,
+        return csound->PerfError(csound, &(p->h),
                                  Str("vtablewk: incorrect table number"));
       }
       p->pfn = (int64_t)*p->xfn;
@@ -440,7 +443,7 @@ static int32_t mtabw_a(CSOUND *csound,MTABW *p)
     if (p->pfn != (int64_t)*p->xfn) {
       FUNC *ftp;
       if (UNLIKELY( (ftp = csound->FTnp2Find(csound, p->xfn) ) == NULL)) {
-        return csound->PerfError(csound, p->h.insdshead,
+        return csound->PerfError(csound, &(p->h),
                                  Str("vtabwa: incorrect table number"));
       }
       p->pfn = (int64_t)*p->xfn;
@@ -465,7 +468,7 @@ static int32_t vectorOp_set(CSOUND *csound, VECTOROP *p)
 {
     FUNC    *ftp;
 
-    ftp = csound->FTnp2Find(csound, p->ifn);
+    ftp = csound->FTnp2Finde(csound, p->ifn);
     if (UNLIKELY(ftp == NULL))
       return NOTOK;
     p->vector = ftp->ftable;
@@ -1725,7 +1728,7 @@ static int32_t vmap_i(CSOUND *csound,VECTORSOPI *p)
 static int32_t vlimit_set(CSOUND *csound,VLIMIT *p)
 {
     FUNC        *ftp;
-    if (UNLIKELY((ftp = csound->FTnp2Find(csound,p->ifn)) != NULL)) {
+    if (UNLIKELY((ftp = csound->FTnp2Finde(csound,p->ifn)) != NULL)) {
       p->vector = ftp->ftable;
       p->elements = (int32_t) *p->ielements;
     }
@@ -2244,7 +2247,7 @@ static int32_t vseg_set(CSOUND *csound,VSEG *p)
       segp++;           /* init each seg ..  */
       curfunc = nxtfunc;
       dur = **argp++;
-      if (UNLIKELY((nxtfunc = csound->FTnp2Find(csound,*argp++)) == NULL))
+      if (UNLIKELY((nxtfunc = csound->FTnp2Finde(csound,*argp++)) == NULL))
         return NOTOK;
       if (dur > FL(0.0)) {
         segp->d = dur * CS_EKR;
@@ -2338,9 +2341,9 @@ static int32_t vphaseseg_set(CSOUND *csound,VPSEG *p)
       /* (segp+nsegs)->cnt = MAXPOS;  */
     }
     argp = p->argums;
-    if ((nxtfunc = csound->FTnp2Find(csound,*argp++)) == NULL)
+    if ((nxtfunc = csound->FTnp2Finde(csound,*argp++)) == NULL)
       return NOTOK;
-    if ((ftp = csound->FTnp2Find(csound,p->ioutfunc)) != NULL) {
+    if ((ftp = csound->FTnp2Finde(csound,p->ioutfunc)) != NULL) {
       p->vector = ftp->ftable;
       p->elements = (int32_t) *p->ielements;
     }
@@ -2362,7 +2365,7 @@ static int32_t vphaseseg_set(CSOUND *csound,VPSEG *p)
       segp++;           /* init each seg ..  */
       curfunc = nxtfunc;
       dur = **argp++;
-      if ((nxtfunc = csound->FTnp2Find(csound,*argp++)) == NULL) return NOTOK;
+      if ((nxtfunc = csound->FTnp2Finde(csound,*argp++)) == NULL) return NOTOK;
       if (dur > FL(0.0)) {
         durtot+=dur;
         segp->d = dur; /* * CS_EKR; */
@@ -2621,15 +2624,15 @@ OENTRY vectorial_localops[] = {
   { "vmap",   S(VECTORSOPI), TB, 1, "",  "iiioo", (SUBR)vmap_i, NULL, NULL  },
   { "vlimit", S(VLIMIT),  TR, 3, "",  "ikki",(SUBR)vlimit_set, (SUBR)vlimit },
   { "vwrap",  S(VLIMIT),  TB, 3, "",  "ikki",(SUBR)vlimit_set, (SUBR) vwrap },
-  { "vmirror", S(VLIMIT),  0,   3, "",  "ikki",(SUBR)vlimit_set, (SUBR)vmirror },
+  { "vmirror", S(VLIMIT),  TB,   3, "",  "ikki",(SUBR)vlimit_set, (SUBR)vmirror },
   { "vlinseg", S(VSEG),   TB, 3, "",  "iim", (SUBR)vseg_set,   (SUBR)vlinseg },
-  { "vexpseg", S(VSEG),    0,   3, "",  "iim", (SUBR)vseg_set, (SUBR)vexpseg },
+  { "vexpseg", S(VSEG),   TB, 3, "",  "iim", (SUBR)vseg_set, (SUBR)vexpseg },
   { "vrandh", S(VRANDH),  TB, 3, "",  "ikkiovoo",(SUBR)vrandh_set, (SUBR)vrandh},
   { "vrandi", S(VRANDI),  TB, 3, "",  "ikkiovoo",(SUBR)vrandi_set, (SUBR)vrandi },
   { "vport",  S(VPORT),   TB, 3, "",  "ikio",(SUBR)vport_set,  (SUBR)vport   },
-  { "vecdelay", S(VECDEL),0,    3, "",  "iiiiio",(SUBR)vecdly_set, (SUBR)vecdly },
-  { "vdelayk", S(KDEL),    0,   3, "k", "kkioo",(SUBR)kdel_set,  (SUBR)kdelay },
-  { "vcella", S(CELLA),      TR, 3, "",  "kkiiiiip",(SUBR)ca_set, (SUBR)ca    }
+  { "vecdelay", S(VECDEL), TB, 3, "",  "iiiiio",(SUBR)vecdly_set, (SUBR)vecdly },
+  { "vdelayk", S(KDEL),    0, 3, "k", "kkioo",(SUBR)kdel_set,  (SUBR)kdelay },
+  { "vcella", S(CELLA),    TB, 3, "",  "kkiiiiip",(SUBR)ca_set, (SUBR)ca    }
 };
 
 int32_t gab_vectorial_init_(CSOUND *csound)
