@@ -40,11 +40,11 @@
 static int32_t syncgrain_init(CSOUND *csound, syncgrain *p)
 {
     int32_t size;
-    p->efunc = csound->FTnp2Find(csound, p->ifn2);
+    p->efunc = csound->FTnp2Finde(csound, p->ifn2);
     if (UNLIKELY(p->efunc == NULL))
       return NOTOK;
 
-    p->sfunc = csound->FTnp2Find(csound, p->ifn1);
+    p->sfunc = csound->FTnp2Finde(csound, p->ifn1);
     if (UNLIKELY(p->sfunc == NULL))
       return NOTOK;
 
@@ -96,15 +96,16 @@ static int32_t syncgrain_process(CSOUND *csound, syncgrain *p)
     int32_t     numstreams = p->numstreams, olaps = p->olaps;
     int32_t     count = p->count, j, newstream;
     int32_t     datasize = p->datasize, envtablesize = p->envtablesize;
+    MYFLT      pscale =  p->sfunc->gen01args.sample_rate/CS_ESR;
 
-    pitch  = *p->pitch;
-    fperiod = FABS(CS_ESR/(*p->fr));
+    pitch  = *p->pitch * pscale;
+    fperiod = FABS(p->sfunc->gen01args.sample_rate/(*p->fr));
     //if (UNLIKELY(fperiod  < 0)) fperiod = -fperiod;
     amp =    *p->amp;
-    grsize = CS_ESR * *p->grsize;
+    grsize = p->sfunc->gen01args.sample_rate * *p->grsize;
     if (UNLIKELY(grsize<1)) goto err1;
     envincr = envtablesize/grsize;
-    prate = *p->prate;
+    prate = *p->prate * pscale;
 
     if (UNLIKELY(offset)) memset(output, '\0', offset*sizeof(MYFLT));
     if (UNLIKELY(early)) {
@@ -186,18 +187,18 @@ static int32_t syncgrain_process(CSOUND *csound, syncgrain *p)
 
     return OK;
  err1:
-    return csound->PerfError(csound, p->h.insdshead,
+    return csound->PerfError(csound, &(p->h),
                              Str("grain size smaller than 1 sample\n"));
 }
 
 
 static int32_t syncgrainloop_init(CSOUND *csound, syncgrainloop *p)
 {
-    p->efunc = csound->FTnp2Find(csound, p->ifn2);
+    p->efunc = csound->FTnp2Finde(csound, p->ifn2);
     if (UNLIKELY(p->efunc == NULL))
       return NOTOK;
 
-    p->sfunc = csound->FTnp2Find(csound, p->ifn1);
+    p->sfunc = csound->FTnp2Finde(csound, p->ifn1);
     if (UNLIKELY(p->sfunc == NULL))
       return NOTOK;
 
@@ -211,18 +212,18 @@ static int32_t syncgrainloop_init(CSOUND *csound, syncgrainloop *p)
     if (*p->iskip == 0) {
       int32_t size =  (p->olaps) * sizeof(double);
       if (p->index.auxp == NULL || p->index.size < (uint32_t)size)
-         csound->AuxAlloc(csound, size, &p->index);
+        csound->AuxAlloc(csound, size, &p->index);
       if (p->envindex.auxp == NULL || p->envindex.size < (uint32_t)size)
-          csound->AuxAlloc(csound, size, &p->envindex);
+        csound->AuxAlloc(csound, size, &p->envindex);
       size = (p->olaps) * sizeof(int32_t);
-       if (p->streamon.auxp == NULL || p->streamon.size > (uint32_t)size)
-          csound->AuxAlloc(csound, size, &p->streamon);
-    p->count = 0;                  /* sampling period counter */
-    p->numstreams = 0;                  /* curr num of streams */
-    p->firststream = 0;                 /* streams index (first stream)  */
-    p->start = *p->startpos*(CS_ESR);
-    p->frac = 0.0f;
-    p->firsttime = 1;
+      if (p->streamon.auxp == NULL || p->streamon.size < (uint32_t)size)
+        csound->AuxAlloc(csound, size, &p->streamon);
+      p->count = 0;                  /* sampling period counter */
+      p->numstreams = 0;                  /* curr num of streams */
+      p->firststream = 0;                 /* streams index (first stream)  */
+      p->start = *p->startpos*(p->sfunc->gen01args.sample_rate);
+      p->frac = 0.0f;
+      p->firsttime = 1;
     }
     return OK;
 }
@@ -248,8 +249,8 @@ static int32_t syncgrainloop_process(CSOUND *csound, syncgrainloop *p)
     int32_t     loop_end;
     int32_t     loopsize;
     int32_t     firsttime = p->firsttime;
-    MYFLT   sr = CS_ESR;
-
+    MYFLT   sr = p->sfunc->gen01args.sample_rate;
+    MYFLT pscale = sr/CS_ESR;
     /* loop points & checks */
     loop_start = (int32_t) (*p->loop_start*sr);
     loop_end = (int32_t) (*p->loop_end*sr);
@@ -260,15 +261,15 @@ static int32_t syncgrainloop_process(CSOUND *csound, syncgrainloop *p)
     /*csound->Message(csound, "st:%d, end:%d, loopsize=%d\n",
                               loop_start, loop_end, loopsize);     */
 
-    pitch  = *p->pitch;
-    fperiod = FABS(CS_ESR/(*p->fr));
+    pitch  = *p->pitch * pscale;
+    fperiod = FABS(sr/(*p->fr));
     //if (UNLIKELY(fperiod  < 0)) fperiod = -fperiod;
     amp =    *p->amp;
-    grsize = CS_ESR * *p->grsize;
+    grsize = sr * *p->grsize;
     if (UNLIKELY(grsize<1)) goto err1;
     if (loopsize <= 0) loopsize = grsize;
     envincr = envtablesize/grsize;
-    prate = *p->prate;
+    prate = *p->prate * pscale;
 
     if (UNLIKELY(offset)) memset(output, '\0', offset*sizeof(MYFLT));
     if (UNLIKELY(early)) {
@@ -359,7 +360,7 @@ static int32_t syncgrainloop_process(CSOUND *csound, syncgrainloop *p)
     p->firsttime = firsttime;
     return OK;
  err1:
-    return csound->PerfError(csound, p->h.insdshead,
+    return csound->PerfError(csound, &(p->h),
                              Str("grain size smaller than 1 sample\n"));
 }
 
@@ -393,6 +394,8 @@ typedef struct _filegrain {
     float   trigger;
     int32_t     nChannels;
     int32   flen;
+    MYFLT pscale;
+    MYFLT sr;
 } filegrain;
 
 #define MINFBUFSIZE  88200
@@ -410,7 +413,7 @@ static int32_t filegrain_init(CSOUND *csound, filegrain *p)
       return csound->InitError(csound,
                                Str("diskgrain: invalid number of channels"));
     }
-    p->efunc = csound->FTnp2Find(csound, p->ifn2);
+    p->efunc = csound->FTnp2Finde(csound, p->ifn2);
     if (UNLIKELY(p->efunc == NULL))
       return NOTOK;
 
@@ -435,6 +438,7 @@ static int32_t filegrain_init(CSOUND *csound, filegrain *p)
                        (p->dataframes+1)*sizeof(MYFLT)*p->nChannels, &p->buffer);
 
     buffer = (MYFLT *) p->buffer.auxp;
+    memset(&sfinfo, '\0', sizeof(sfinfo)); /* for Valgrind */
     /* open file and read the first block using *p->ioff */
     fd = csound->FileOpen2(csound, &(p->sf), CSFILE_SND_R, fname, &sfinfo,
                             "SFDIR;SSDIR", CSFTYPE_UNKNOWN_AUDIO, 0);
@@ -449,8 +453,11 @@ static int32_t filegrain_init(CSOUND *csound, filegrain *p)
                                       "do not match the number of outputs\n"));
     }
 
+    p->sr = sfinfo.samplerate;
+    p->pscale = p->sr/CS_ESR;
+
     if (*p->ioff >= 0)
-      sf_seek(p->sf,*p->ioff * CS_ESR, SEEK_SET);
+      sf_seek(p->sf,*p->ioff * p->sr, SEEK_SET);
 
     if (LIKELY(sf_read_MYFLT(p->sf,buffer,p->dataframes*p->nChannels/2) != 0)) {
       p->read1 = 1;
@@ -468,7 +475,7 @@ static int32_t filegrain_init(CSOUND *csound, filegrain *p)
 
     p->start = 0.0f;
     p->frac = 0.0f;
-    p->pos = *p->ioff*CS_ESR;
+    p->pos = *p->ioff*p->sr;
     p->trigger = 0.0f;
     p->flen = sfinfo.frames;
 
@@ -505,15 +512,15 @@ static int32_t filegrain_process(CSOUND *csound, filegrain *p)
     datasize = dataframes*chans;
     hdatasize = hdataframes*chans;
 
-    pitch  = *p->pitch;
-    fperiod = FABS(CS_ESR/(*p->fr));
+    pitch  = *p->pitch * p->pscale;
+    fperiod = FABS(p->sr/(*p->fr));
     //if (UNLIKELY(fperiod  < FL(0.0))) fperiod = -fperiod;
     amp =    *p->amp;
-    grsize = CS_ESR * *p->grsize;
+    grsize = p->sr * *p->grsize;
     if (UNLIKELY(grsize<1)) goto err1;
     if (grsize > hdataframes) grsize = hdataframes;
     envincr = envtablesize/grsize;
-    prate = *p->prate;
+    prate = *p->prate * p->pscale;
 
     if (UNLIKELY(offset)) memset(output, '\0', offset*sizeof(MYFLT));
     if (UNLIKELY(early)) {
@@ -710,7 +717,7 @@ static int32_t filegrain_process(CSOUND *csound, filegrain *p)
     p->pos = pos;
     return OK;
  err1:
-    return csound->PerfError(csound, p->h.insdshead,
+    return csound->PerfError(csound, &(p->h),
                              Str("grain size smaller than 1 sample\n"));
 }
 
