@@ -17,8 +17,8 @@
 
     You should have received a copy of the GNU Lesser General Public
     License along with Csound; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-    02111-1307 USA
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+    02110-1301 USA
 */
 #include "csoundCore.h"         /*                      ARGDECODE.C     */
 #include "soundio.h"
@@ -255,6 +255,9 @@ static const char *longUsageList[] = {
                                    "ORC/SCO-relative line #s"),
   Str_noop("--extract-score=FNAME   extract from score.srt using extract file"),
   Str_noop("--keep-sorted-score"),
+  Str_noop("--keep-sorted-score=FNAME"),
+  Str_noop("--simple-sorted-score"),
+  Str_noop("--simple-sorted-score=FNAME"),
   Str_noop("--env:NAME=VALUE        set environment variable NAME to VALUE"),
   Str_noop("--env:NAME+=VALUE       append VALUE to environment variable NAME"),
   Str_noop("--strsetN=VALUE         set strset table at index N to VALUE"),
@@ -268,6 +271,7 @@ static const char *longUsageList[] = {
   Str_noop("--sched                 set real-time scheduling priority and "
                                    "lock memory"),
   Str_noop("--sched=N               set priority to N and lock memory"),
+  Str_noop("--opcode-dir=DIR        load all plugins from DIR"),
   Str_noop("--opcode-lib=NAMES      dynamic libraries to load"),
   Str_noop("--opcode-omit=NAMES     dynamic libraries not to load"),
   Str_noop("--omacro:XXX=YYY        set orchestra macro XXX to value YYY"),
@@ -298,9 +302,12 @@ static const char *longUsageList[] = {
   Str_noop("--port=N                listen to UDP port N for instruments/orchestra "
                                     "code (implies --daemon)"),
   Str_noop("--vbr-quality=Ft        set quality of variable bit-rate compression"),
-  Str_noop("--devices[=in|out]      list available MIDI devices and exit"),
-  Str_noop("--midi-devices[=in|out] list available audio devices and exit"),
-  Str_noop("--get-system-sr         print system sr and exit"),
+  Str_noop("--devices[=in|out]      list available audio devices and exit"),
+  Str_noop("--midi-devices[=in|out] list available MIDI devices and exit"),
+  Str_noop("--get-system-sr         print system sr and exit, requires realtime\n"
+       "                        audio output (e.g. -odac) to be defined first)"),
+  Str_noop("--use-system-sr         print system sr and use realtime audio\n"
+           "                        output (e.g. -odac) to be defined first"),
   Str_noop("--ksmps=N               override ksmps"),
   Str_noop("--fftlib=N              actual FFT lib to use (FFTLIB=0, "
                                    "PFFFT = 1, vDSP =2)"),
@@ -349,11 +356,11 @@ CS_NORETURN void dieu(CSOUND *csound, char *s, ...)
     va_start(args, s);
     csound->ErrMsgV(csound, Str("Csound Command ERROR:    "), s, args);
     va_end(args);
-    //***FIXME This code makes no sense
-    if (csound->info_message_request == 0) {
-      csound->info_message_request = 0;
-      csound->LongJmp(csound, 1);
-    }
+    //// FIXME This code makes no sense
+    /* if (csound->info_message_request == 0) { */
+    /*   csound->info_message_request = 0; */
+    /*   csound->LongJmp(csound, 1); */
+    /* } */
     //Added longjump -- JPff
     csound->LongJmp(csound, 1);
 }
@@ -506,7 +513,7 @@ static int decode_long(CSOUND *csound, char *s, int argc, char **argv)
       nn->mac = s;
       nn->next = csound->omacros;
       csound->omacros = nn;
-      return 1;
+     return 1;
     }
     else if (!(strncmp(s, "smacro:", 7))) {
       if (csound->orcname_mode) return 1;
@@ -565,7 +572,7 @@ static int decode_long(CSOUND *csound, char *s, int argc, char **argv)
       if (UNLIKELY(*s=='\0')) dieu(csound, Str("no hardware bufsamps"));
       O->inbufsamps = O->outbufsamps = atoi(s);
       return 1;
-    }
+   }
     else if (!(strcmp (s, "orc"))) {
       csound->use_only_orchfile = 1;    /* orchfile without scorefile */
       return 1;
@@ -845,6 +852,10 @@ static int decode_long(CSOUND *csound, char *s, int argc, char **argv)
       O->sfwrite = 1;
       return 1;
     }
+    else if (!(strcmp (s, "print_version"))) {
+      csound->print_version = 1;
+      return 1;
+    }
     else if (!(strncmp (s, "logfile=", 8))) {
       s += 8;
       if (UNLIKELY(*s=='\0')) dieu(csound, Str("no log file"));
@@ -873,8 +884,24 @@ static int decode_long(CSOUND *csound, char *s, int argc, char **argv)
       return 1;
     }
     /* -t0 */
+    else if (!(strncmp(s, "keep-sorted-score=", 18))) {
+      s += 18;
+      csound->score_srt = s;
+      csound->keep_tmp= 1;
+      return 1;
+    }
     else if (!(strcmp (s, "keep-sorted-score"))) {
-      csound->keep_tmp = 1;
+      csound->keep_tmp= 1;
+      return 1;
+    }
+    else if (!(strncmp (s, "simple-sorted-score=", 20))) {
+      s +=20;
+      csound->score_srt = s;
+      csound->keep_tmp = 2;
+      return 1;
+    }
+    else if (!(strcmp (s, "simple-sorted-score"))) {
+      csound->keep_tmp = 2;
       return 1;
     }
     /* IV - Jan 27 2005: --expression-opt */
@@ -989,6 +1016,11 @@ static int decode_long(CSOUND *csound, char *s, int argc, char **argv)
       O->midiVelocityAmp = atoi(s);
       return 1;
     }
+    else if (!(strncmp (s, "opcode-dir=", 11))){
+        s += 11;
+        csoundLoadPlugins(csound, s);
+        return 1;
+    }
     else if (!(strncmp (s, "opcode-lib=", 11))) {
       int   nbytes;
       s += 11;
@@ -1080,7 +1112,7 @@ static int decode_long(CSOUND *csound, char *s, int argc, char **argv)
       return 1;  /* Try new parser */
     }
     else if (!(strcmp(s, "daemon"))) {
-      O->daemon = 1;
+      if(O->daemon == 0) O->daemon = 1;
       return 1;
     }
     else if (!(strncmp(s, "port=",5))) {
@@ -1161,7 +1193,7 @@ static int decode_long(CSOUND *csound, char *s, int argc, char **argv)
       csound->info_message_request = 1;
       return 1;
       }
-    else if (!(strncmp(s, "get-system-sr",13))){
+    else if (!(strncmp(s, "get-system-sr",13)) ){
       if (O->outfilename &&
           !(strncmp(O->outfilename, "dac",3))) {
         /* these are default values to get the
@@ -1180,6 +1212,11 @@ static int decode_long(CSOUND *csound, char *s, int argc, char **argv)
       csound->info_message_request = 1;
       return 1;
     }
+    else if(!strncmp(s, "use-system-sr",13)){
+      if (O->sr_override == FL(0.0))
+          O->sr_override = FL(-1.0);
+      return 1;
+    }
     else if (!(strcmp(s, "aft-zero"))){
       csound->aftouch = 0;
       return 1;
@@ -1212,7 +1249,7 @@ PUBLIC int argdecode(CSOUND *csound, int argc, const char **argv_)
       strcpy(p2, argv_[i]);
       p2 = (char*) p2 + ((int) strlen(argv_[i]) + 1);
     }
-    csound->keep_tmp = 0;
+    //<csound->keep_tmp = 0;
 
     do {
       s = *++argv;
@@ -1237,6 +1274,7 @@ PUBLIC int argdecode(CSOUND *csound, int argc, const char **argv_)
           case 'I':
             csound->initonly = 1;         /* I-only overrides */
             O->syntaxCheckOnly = 0;       /* --syntax-check-only and implies */
+            /* FALLTHRU */
           case 'n':
             O->sfwrite = 0;               /* nosound        */
             break;
@@ -1799,10 +1837,10 @@ static void list_audio_devices(CSOUND *csound, int output){
       csound->Malloc(csound, n*sizeof(CS_AUDIODEVICE));
     if (output)
       csound->MessageS(csound,CSOUNDMSG_STDOUT,
-                       Str("%d audio output devices \n"), n);
+                       Str("%d audio output devices\n"), n);
     else
       csound->MessageS(csound, CSOUNDMSG_STDOUT,
-                       Str("%d audio input devices \n"), n);
+                       Str("%d audio input devices\n"), n);
     csoundGetAudioDevList(csound,devs,output);
     for (i=0; i < n; i++)
       csound->Message(csound, " %d: %s (%s)\n",
@@ -1817,10 +1855,10 @@ static void list_midi_devices(CSOUND *csound, int output){
       (CS_MIDIDEVICE *) csound->Malloc(csound, n*sizeof(CS_MIDIDEVICE));
     if (output)
       csound->MessageS(csound, CSOUNDMSG_STDOUT,
-                       Str("%d MIDI output devices \n"), n);
+                       Str("%d MIDI output devices\n"), n);
     else
       csound->MessageS(csound, CSOUNDMSG_STDOUT,
-                       Str("%d MIDI input devices \n"), n);
+                       Str("%d MIDI input devices\n"), n);
     csoundGetMIDIDevList(csound,devs,output);
     for (i=0; i < n; i++)
       csound->Message(csound, " %d: %s (%s)\n",
