@@ -89,7 +89,7 @@
 #endif
 
 #if !(defined (NACL))
-#if defined(LINUX) || defined(NEW_MACH_CODE)
+#if defined(LINUX) || defined(NEW_MACH_CODE) || defined(__HAIKU__)
 #include <dlfcn.h>
 #elif defined(WIN32)
 #include <windows.h>
@@ -136,7 +136,20 @@ static  const   char    *plugindir64_envvar = "OPCODE6DIR64";
 
 /* default directory to load plugins from if environment variable is not set */
 #if !(defined (NACL))
-#if !(defined(_CSOUND_RELEASE_) && (defined(LINUX) || defined(__MACH__)))
+#ifdef __HAIKU__
+# ifndef USE_DOUBLE
+   static char haikudirs[] = "/boot/system/lib/csound6/plugins:"
+        "/boot/home/config/lib/csound6/plugins:"
+        "/boot/system/non-packaged/lib/csound6/plugins:"
+        "/boot/home/config/non-packaged/lib/csound6/plugins";
+# else
+   static char haikudirs[] = "/boot/system/lib/csound6/plugins64:"
+        "/boot/home/config/lib/csound6/plugins64:"
+        "/boot/system/non-packaged/lib/csound6/plugins64:"
+        "/boot/home/config/non-packaged/lib/csound6/plugins64";
+# endif
+# define CS_DEFAULT_PLUGINDIR  haikudirs
+#elif !(defined(_CSOUND_RELEASE_) && (defined(LINUX) || defined(__MACH__)))
 #  define ENABLE_OPCODEDIR_WARNINGS 1
 #  ifdef CS_DEFAULT_PLUGINDIR
 #    undef CS_DEFAULT_PLUGINDIR
@@ -251,7 +264,7 @@ static CS_NOINLINE int csoundLoadExternal(CSOUND *csound,
     err = csoundOpenLibrary(&h, libraryPath);
     if (UNLIKELY(err)) {
       char ERRSTR[256];
- #if !(defined(NACL)) && defined(LINUX)
+ #if !(defined(NACL)) && (defined(LINUX) || defined(__HAIKU__))
       snprintf(ERRSTR, 256, Str("could not open library '%s' (%s)"),
                libraryPath, dlerror());
  #else
@@ -411,6 +424,9 @@ int csoundLoadModules(CSOUND *csound)
 #else
     ':';
 #endif
+#ifdef __HAIKU__
+        int dfltdir = 0;
+#endif
 
     if (UNLIKELY(csound->csmodule_db != NULL))
       return CSOUND_ERROR;
@@ -428,10 +444,18 @@ int csoundLoadModules(CSOUND *csound)
 #endif
 #ifdef  CS_DEFAULT_PLUGINDIR
         dname = CS_DEFAULT_PLUGINDIR;
+ #ifdef __HAIKU__
+                dfltdir = 1;
+ #endif
 #else
       dname = "";
 #endif
     }
+    /* opcodedir GLOBAL override **experimental** */
+    if (csound->opcodedir != NULL) {
+      dname = csound->opcodedir;
+      csound->Message(csound, "OPCODEDIR overriden to %s \n", dname);
+    }
 
     /* We now loop through the directory list */
     while(read_directory) {
@@ -440,6 +464,7 @@ int csoundLoadModules(CSOUND *csound)
       *end = '\0';
       /* copy directory name */
       dname1 = cs_strdup(csound, (char *) dname);
+      *end = sep;  /* restore for re-execution */
       /* move to next directory name */
       dname = end + 1;
     } else {
@@ -457,6 +482,9 @@ int csoundLoadModules(CSOUND *csound)
 
     dir = opendir(dname1);
     if (UNLIKELY(dir == (DIR*) NULL)) {
+ #if defined(__HAIKU__)
+        if(!dfltdir)
+ #endif
       csound->Warning(csound, Str("Error opening plugin directory '%s': %s"),
                                dname1, strerror(errno));
       csound->Free(csound, dname1);
@@ -505,7 +533,7 @@ int csoundLoadModules(CSOUND *csound)
         csoundWarning(csound, Str("Library %s omitted\n"), fname);
         continue;
       }
-      snprintf(buf, 1024, "%s%c%s", dname, DIRSEP, fname);
+      snprintf(buf, 1024, "%s%c%s", dname1, DIRSEP, fname);
       if (UNLIKELY(csound->oparms->odebug)) {
         csoundMessage(csound, Str("Loading '%s'\n"), buf);
        }
@@ -524,6 +552,7 @@ int csoundLoadModules(CSOUND *csound)
 #endif  /* HAVE_DIRENT_H */
 }
 
+
 static int cmp_func(const void *p1, const void *p2)
 {
     return (strcmp(*((const char**) p1), *((const char**) p2)));
@@ -671,6 +700,149 @@ int csoundLoadAndInitModule(CSOUND *csound, const char *fname)
     return err;
 }
 
+int csoundLoadAndInitModules(CSOUND *csound, const char *opdir)
+{
+#if (defined(HAVE_DIRENT_H) && (TARGET_OS_IPHONE == 0))
+    DIR             *dir;
+    struct dirent   *f;
+    const char      *dname, *fname;
+    char            buf[1024];
+    int             i, n, len, err = CSOUND_SUCCESS;
+    char   *dname1, *end;
+    int     read_directory = 1;
+    char sep =
+#ifdef WIN32
+    ';';
+#else
+    ':';
+#endif
+#ifdef __HAIKU__
+        int dfltdir = 0;
+#endif
+
+    if (UNLIKELY(csound->csmodule_db != NULL))
+      return CSOUND_ERROR;
+
+    /* open plugin directory */
+    dname = csoundGetEnv(csound, (sizeof(MYFLT) == sizeof(float) ?
+                                  plugindir_envvar : plugindir64_envvar));
+    if (dname == NULL) {
+#if ENABLE_OPCODEDIR_WARNINGS
+      csound->opcodedirWasOK = 0;
+#  ifdef USE_DOUBLE
+      dname = csoundGetEnv(csound, plugindir_envvar);
+      if (dname == NULL)
+#  endif
+#endif
+#ifdef  CS_DEFAULT_PLUGINDIR
+        dname = CS_DEFAULT_PLUGINDIR;
+ #ifdef __HAIKU__
+                dfltdir = 1;
+ #endif
+#else
+      dname = "";
+#endif
+    }
+
+    if (opdir != NULL) {
+      dname = opdir;
+    }
+
+    /* We now loop through the directory list */
+    while(read_directory) {
+      /* find separator */
+    if((end = strchr(dname, sep)) != NULL) {
+      *end = '\0';
+      /* copy directory name */
+      dname1 = cs_strdup(csound, (char *) dname);
+      *end = sep;  /* restore for re-execution */
+      /* move to next directory name */
+      dname = end + 1;
+    } else {
+      /* copy last directory name) */
+      dname1 = cs_strdup(csound, (char *) dname);
+      read_directory = 0;
+    }
+
+    /* protect for the case where there is an
+       extra separator at the end */
+    if(*dname1 == '\0') {
+      csound->Free(csound, dname1);
+      break;
+    }
+
+    dir = opendir(dname1);
+    if (UNLIKELY(dir == (DIR*) NULL)) {
+ #if defined(__HAIKU__)
+        if(!dfltdir)
+ #endif
+      csound->Warning(csound, Str("Error opening plugin directory '%s': %s"),
+                               dname1, strerror(errno));
+      csound->Free(csound, dname1);
+      continue;
+    }
+
+    if(UNLIKELY(csound->oparms->odebug))
+      csound->Message(csound, "Opening plugin directory: %s\n", dname1);
+    /* load database for deferred plugin loading */
+/*     n = csoundLoadOpcodeDB(csound, dname); */
+/*     if (n != 0) */
+/*       return n; */
+    /* scan all files in directory */
+    while ((f = readdir(dir)) != NULL) {
+      fname = &(f->d_name[0]);
+      if (UNLIKELY(fname[0]=='_')) continue;
+      n = len = (int) strlen(fname);
+      if (UNLIKELY(fname[0]=='_')) continue;
+#if defined(WIN32)
+      strcpy(buf, "dll");
+      n -= 4;
+#elif defined(__MACH__)
+      strcpy(buf, "dylib");
+      n -= 6;
+#else
+      strcpy(buf, "so");
+      n -= 3;
+#endif
+      if (n <= 0 || fname[n] != '.')
+        continue;
+      i = 0;
+      do {
+        if (UNLIKELY((fname[++n] | (char) 0x20) != buf[i]))
+          break;
+      } while (buf[++i] != '\0');
+      if (buf[i] != '\0')
+        continue;
+      /* found a dynamic library, attempt to open it */
+      if (UNLIKELY(((int) strlen(dname) + len + 2) > 1024)) {
+        csound->Warning(csound, Str("path name too long, skipping '%s'"),
+                                fname);
+        continue;
+      }
+      /* printf("DEBUG %s(%d): possibly deny %s\n", __FILE__, __LINE__,fname); */
+      if (UNLIKELY(csoundCheckOpcodeDeny(csound, fname))) {
+        csoundWarning(csound, Str("Library %s omitted\n"), fname);
+        continue;
+      }
+      snprintf(buf, 1024, "%s%c%s", dname1, DIRSEP, fname);
+      if (UNLIKELY(csound->oparms->odebug)) {
+        csoundMessage(csound, Str("Loading '%s'\n"), buf);
+       }
+      n = csoundLoadAndInitModule(csound, buf);
+      if (UNLIKELY(UNLIKELY(n == CSOUND_ERROR)))
+        continue;               /* ignore non-plugin files */
+      if (UNLIKELY(n < err))
+        err = n;                /* record serious errors */
+    }
+    closedir(dir);
+    csound->Free(csound, dname1);
+    }
+    return (err == CSOUND_INITIALIZATION ? CSOUND_ERROR : err);
+#else
+    return CSOUND_SUCCESS;
+#endif  /* HAVE_DIRENT_H */
+}
+
 /**
  * Call destructor functions of all loaded modules that have a
  * csoundModuleDestroy symbol, for Csound instance 'csound'.
@@ -728,7 +900,7 @@ PUBLIC void *csoundGetLibrarySymbol(void *library, const char *procedureName)
     return (void*) GetProcAddress((HMODULE) library, procedureName);
 }
 
-#elif !(defined(NACL)) && (defined(LINUX) || defined (NEW_MACH_CODE))
+#elif !(defined(NACL)) && (defined(LINUX) || defined(NEW_MACH_CODE) || defined(__HAIKU__))
 
 PUBLIC int csoundOpenLibrary(void **library, const char *libraryPath)
 {
@@ -829,7 +1001,7 @@ extern long hrtfopcodes_localops_init(CSOUND *, void *);
 extern long hrtfreverb_localops_init(CSOUND *, void *);
 extern long hrtfearly_localops_init(CSOUND *, void *);
 extern long minmax_localops_init(CSOUND *, void *);
-
+extern long gendy_localops_init(CSOUND *, void *);
 //extern long stackops_localops_init(CSOUND *, void *);
 extern long vbap_localops_init(CSOUND *, void *);
 extern long vaops_localops_init(CSOUND *, void*);
@@ -840,6 +1012,7 @@ extern long squinewave_localops_init(CSOUND *, void *);
 
 extern long partikkel_localops_init(CSOUND *, void *);
 extern long shape_localops_init(CSOUND *, void *);
+extern long tabaudio_localops_init(CSOUND *, void *);
 extern long tabsum_localops_init(CSOUND *, void *);
 extern long crossfm_localops_init(CSOUND *, void *);
 extern long pvlock_localops_init(CSOUND *, void *);
@@ -855,6 +1028,8 @@ extern long afilts_localops_init(CSOUND *, void *);
 extern long pinker_localops_init(CSOUND *, void *);
 extern long paulstretch_localops_init(CSOUND *, void *);
 extern long wpfilters_localops_init(CSOUND *, void *);
+extern long zak_localops_init(CSOUND *, void *);
+extern long lufs_localops_init(CSOUND *, void *);
 
 extern int stdopc_ModuleInit(CSOUND *csound);
 extern int pvsopc_ModuleInit(CSOUND *csound);
@@ -872,7 +1047,7 @@ const INITFN staticmodules[] = { hrtfopcodes_localops_init, babo_localops_init,
                                  hrtferX_localops_init, loscilx_localops_init,
                                  pan2_localops_init, arrayvars_localops_init,
                                  phisem_localops_init, pvoc_localops_init,
-                                 /*stackops_localops_init,*/ vbap_localops_init,
+                                 vbap_localops_init,
                                  ugakbari_localops_init, harmon_localops_init,
                                  pitchtrack_localops_init, partikkel_localops_init,
                                  shape_localops_init, tabsum_localops_init,
@@ -880,7 +1055,7 @@ const INITFN staticmodules[] = { hrtfopcodes_localops_init, babo_localops_init,
                                  fareyseq_localops_init, hrtfearly_localops_init,
                                  hrtfreverb_localops_init, minmax_localops_init,
                                  vaops_localops_init, paulstretch_localops_init,
-                                 squinewave_localops_init,
+                                 squinewave_localops_init, tabaudio_localops_init,
 #ifdef LINUX
                                  cpumeter_localops_init,
 #endif
@@ -890,8 +1065,9 @@ const INITFN staticmodules[] = { hrtfopcodes_localops_init, babo_localops_init,
                                  socksend_localops_init,
 #endif
                                  scnoise_localops_init, afilts_localops_init,
-                                 pinker_localops_init,
-                                 wpfilters_localops_init,
+                                 pinker_localops_init, gendy_localops_init,
+                                 wpfilters_localops_init, zak_localops_init,
+                                 lufs_localops_init,
                                  NULL };
 
 typedef NGFENS* (*FGINITFN)(CSOUND *);
