@@ -182,7 +182,7 @@ PUBLIC void *csoundCreateThread(uintptr_t (*threadRoutine)(void *),
 {
     pthread_t *pthread = (pthread_t *) malloc(sizeof(pthread_t));
     if (!pthread_create(pthread, (pthread_attr_t*) NULL,
-                        (void *(*)(void *)) threadRoutine, userdata)) {
+                        (void *(*)(void *))(void*)threadRoutine, userdata)) {
       return (void*) pthread;
     }
     free(pthread);
@@ -205,6 +205,7 @@ PUBLIC uintptr_t csoundJoinThread(void *thread)
     if(thread == NULL) return 0;
     pthreadReturnValue = pthread_join(*pthread,
                                       &threadRoutineReturnValue);
+    free(pthread);
     if (pthreadReturnValue) {
         return (uintptr_t) ((intptr_t) pthreadReturnValue);
     } else {
@@ -556,6 +557,11 @@ PUBLIC void csoundCondSignal(void* condVar) {
         pthread_cond_signal(condVar);
 }
 
+PUBLIC void csoundDestroyCondVar(void* condVar) {
+        pthread_cond_destroy((pthread_cond_t*)condVar);
+        free(condVar);
+}
+
 /* ------------------------------------------------------------------------ */
 
 #elif defined(WIN32)
@@ -781,6 +787,11 @@ PUBLIC void csoundCondSignal(void* condVar) {
     WakeConditionVariable(cv);
 }
 
+PUBLIC void csoundDestroyCondVar(void* condVar) {
+    memset(condVar, '\0', sizeof(CONDITION_VARIABLE));
+    free(condVar);
+}
+
 // REMOVE FOLLOWING BARRIER DEFINITION WINDOWS SUPPORT LIMITED to WIN 8.1+
 typedef struct barrier {
     CRITICAL_SECTION* mut;
@@ -948,25 +959,29 @@ PUBLIC int csoundWaitBarrier(void *barrier)
 
 PUBLIC void* csoundCreateCondVar()
 {
-  //notImplementedWarning_("csoundCreateCondVar");
-  return NULL;
+    //notImplementedWarning_("csoundCreateCondVar");
+    return NULL;
 }
 
 PUBLIC void csoundCondWait(void* condVar, void* mutex) {
-  //notImplementedWarning_("csoundCreateCondWait");
+    //notImplementedWarning_("csoundCreateCondWait");
 }
 
 PUBLIC void csoundCondSignal(void* condVar) {
- // notImplementedWarning_("csoundCreateCondSignal");
+    // notImplementedWarning_("csoundCreateCondSignal");
+}
+
+PUBLIC void csoundDestroyCondVar(void* condVar) {
+    // notImplementedWarning_("csoundDestroyCondVar");
 }
 
 PUBLIC long csoundRunCommand(const char * const *argv, int noWait) {
-  //notImplementedWarning_("csoundRunCommand");
+    //notImplementedWarning_("csoundRunCommand");
     return 0;
 }
 
 PUBLIC void csoundSleep(size_t milliseconds) {
-  //notImplementedWarning_("csoundSleep");
+    //notImplementedWarning_("csoundSleep");
 }
 
 
@@ -977,60 +992,20 @@ PUBLIC void csoundSleep(size_t milliseconds) {
 /* This pragma must come before all public function declarations */
 # pragma intrinsic(_InterlockedExchange)
 void csoundSpinLock(spin_lock_t *spinlock) {
-  while (_InterlockedExchange(spinlock, 1) == 1){};
+    while (_InterlockedExchange(spinlock, 1) == 1){};
 }
 
 void csoundSpinUnLock(spin_lock_t *spinlock){
-      _InterlockedExchange(spinlock, 0);
+    _InterlockedExchange(spinlock, 0);
 }
 
 int csoundSpinTryLock(spin_lock_t *spinlock) {
-  return _InterlockedExchange(spinlock, 1) == 0 ? CSOUND_SUCCESS : CSOUND_ERROR;
+    return _InterlockedExchange(spinlock, 1) == 0 ? CSOUND_SUCCESS : CSOUND_ERROR;
 }
 
 int csoundSpinLockInit(spin_lock_t *spinlock) {
-  *spinlock = SPINLOCK_INIT;
-  return 0;
-}
-
-#elif defined(__GNUC__) && defined(HAVE_PTHREAD_SPIN_LOCK)
-// POSIX spin locks
-
-void csoundSpinLock(spin_lock_t *spinlock) {
-  pthread_spin_lock(spinlock);
-}
-
-void csoundSpinUnLock(spin_lock_t *spinlock){
-  pthread_spin_unlock(spinlock);
-}
-
-int csoundSpinTryLock(spin_lock_t *spinlock) {
-  return pthread_spin_trylock(spinlock);
-}
-
-int csoundSpinLockInit(spin_lock_t *spinlock) {
-  return pthread_spin_init(spinlock, PTHREAD_PROCESS_PRIVATE);
-}
-
-
-#elif defined(HAVE_BUILTIN_ATOMIC)
-// No POSIX spinlocks but GCC intrinsics
-
-void csoundSpinLock(spin_lock_t *spinlock){
-  while (__sync_lock_test_and_set(spinlock, 1) == 1) { };
-}
-
-void csoundSpinUnLock(spin_lock_t *spinlock){
-      __sync_lock_release(spinlock);
-}
-
-int csoundSpinTryLock(spin_lock_t *spinlock) {
-  return __sync_lock_test_and_set(spinlock, 1) == 0 ? CSOUND_SUCCESS : CSOUND_ERROR;
-}
-
-int csoundSpinLockInit(spin_lock_t *spinlock) {
-  *spinlock = SPINLOCK_INIT;
-  return 0;
+    *spinlock = SPINLOCK_INIT;
+    return 0;
 }
 
 #elif defined(MACOSX) // MacOS native locks
@@ -1039,59 +1014,108 @@ int csoundSpinLockInit(spin_lock_t *spinlock) {
 // New spinlock interface
 
 void csoundSpinLock(spin_lock_t *spinlock){
-      os_unfair_lock_lock(spinlock);
+    os_unfair_lock_lock(spinlock);
 }
 
 void csoundSpinUnLock(spin_lock_t *spinlock){
-      os_unfair_lock_unlock(spinlock);
+    os_unfair_lock_unlock(spinlock);
 }
 
 int csoundSpinTryLock(spin_lock_t *spinlock) {
-  return os_unfair_lock_trylock(spinlock) == true ? CSOUND_SUCCESS : CSOUND_ERROR;
+    return os_unfair_lock_trylock(spinlock) == true ? CSOUND_SUCCESS : CSOUND_ERROR;
 }
 
 int csoundSpinLockInit(spin_lock_t *spinlock) {
-  IGN(spinlock);
-  return 0;
+    IGN(spinlock);
+    return 0;
 }
 
 #else // Old spinlock interface
 
 void csoundSpinLock(spin_lock_t *spinlock) {
-      OSSpinLockLock(spinlock);
+    OSSpinLockLock((volatile OSSpinLock *) spinlock);
 }
 
 void csoundSpinUnLock(spin_lock_t *spinlock) {
-      OSSpinLockUnlock(spinlock);
+    OSSpinLockUnlock((volatile OSSpinLock *) spinlock);
 }
 
 int csoundSpinTryLock(spin_lock_t *spinlock) {
-  return OSSpinLockTry(spinlock) == true ? CSOUND_SUCCESS : CSOUND_ERROR;
+    return OSSpinLockTry((volatile OSSpinLock *) spinlock) == true ?
+      CSOUND_SUCCESS : CSOUND_ERROR;
 }
 
 int csoundSpinLockInit(spin_lock_t *spinlock) {
-  *spinlock = SPINLOCK_INIT;
-  return 0;
+    *spinlock = SPINLOCK_INIT;
+    return 0;
 }
 
 #endif // MAC_OS_X_VERSION_MIN_REQUIRED
 
-#else // No spinlocks
+#elif defined(__GNUC__) && defined(HAVE_PTHREAD_SPIN_LOCK)
+// POSIX spin locks
+
 void csoundSpinLock(spin_lock_t *spinlock) {
-      IGN(spinlock);
+    pthread_spin_lock(spinlock);
 }
-void csoundSpinUnLock(spin_lock_t *spinlock) {
-      IGN(spinlock);
+
+void csoundSpinUnLock(spin_lock_t *spinlock){
+    pthread_spin_unlock(spinlock);
 }
 
 int csoundSpinTryLock(spin_lock_t *spinlock) {
-  IGN(spinlock);
-  return 1;
+    return pthread_spin_trylock(spinlock);
 }
 
 int csoundSpinLockInit(spin_lock_t *spinlock) {
-  IGN(spinlock);
-  return 0;
+    return pthread_spin_init(spinlock, PTHREAD_PROCESS_PRIVATE);
+}
+
+
+#elif defined(__GNUC__) && defined(HAVE_ATOMIC_BUILTIN)
+// No POSIX spinlocks but GCC intrinsics
+#include <stdbool.h>
+
+void csoundSpinLock(spin_lock_t *spinlock){
+    spin_lock_t unset = 0;
+    spin_lock_t set = 1;
+    while (!__atomic_compare_exchange_n(spinlock, &unset, set, false,
+                                        __ATOMIC_SEQ_CST, __ATOMIC_SEQ_CST)) { };
+}
+
+void csoundSpinUnLock(spin_lock_t *spinlock){
+    __atomic_clear(spinlock, __ATOMIC_SEQ_CST);
+}
+
+int csoundSpinTryLock(spin_lock_t *spinlock) {
+    spin_lock_t unset = 0;
+    spin_lock_t set = 1;
+    return __atomic_compare_exchange_n(spinlock, &unset, set, false,
+                                       __ATOMIC_SEQ_CST, __ATOMIC_SEQ_CST) ?
+      CSOUND_SUCCESS : CSOUND_ERROR;
+}
+
+int csoundSpinLockInit(spin_lock_t *spinlock) {
+    *spinlock = SPINLOCK_INIT;
+    return 0;
+}
+
+#else // No spinlocks
+void csoundSpinLock(spin_lock_t *spinlock) {
+    IGN(spinlock);
+}
+void csoundSpinUnLock(spin_lock_t *spinlock) {
+    IGN(spinlock);
+}
+
+int csoundSpinTryLock(spin_lock_t *spinlock) {
+    IGN(spinlock);
+    return 1;
+}
+
+int csoundSpinLockInit(spin_lock_t *spinlock) {
+    IGN(spinlock);
+    return 0;
 }
 
 #endif
