@@ -17,8 +17,8 @@
 
     You should have received a copy of the GNU Lesser General Public
     License along with Csound; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-    02111-1307 USA
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+    02110-1301 USA
 */
 
 #include <iostream>
@@ -595,7 +595,12 @@ CsoundPerformanceThread::~CsoundPerformanceThread()
     }
     if (recordLock) {
         csoundDestroyMutex(recordLock);
-
+    }
+    if (recordData.mutex) {
+        csoundDestroyMutex(recordData.mutex);
+    }
+    if (recordData.condvar) {
+        csoundDestroyCondVar(recordData.condvar);
     }
 }
 
@@ -676,15 +681,15 @@ int CsoundPerformanceThread::Join()
     int retval;
     retval = status;
 
-    if (perfThread) {
-      retval = csoundJoinThread(perfThread);
-      perfThread = (void*) 0;
-    }
     if (recordData.running) {
         recordData.running = false;
         csoundCondSignal(recordData.condvar);
         csoundJoinThread(recordData.thread);
     }
+    if (perfThread) {
+      retval = csoundJoinThread(perfThread);
+      perfThread = (void*) 0;
+    }
 
     // delete any pending messages
     {
